from __future__ import unicode_literals

from spacy.en import English

import pytest

@pytest.mark.models
def test_vec(EN):
    hype = EN.vocab['hype']
    assert hype.orth_ == 'hype'
    assert 0.08 >= hype.vector[0] > 0.07


@pytest.mark.models
def test_capitalized(EN):
    hype = EN.vocab['Hype']
    assert hype.orth_ == 'Hype'
    assert 0.08 >= hype.vector[0] > 0.07



import plac
from ..download import download


@plac.annotations(
    force=("Force overwrite", "flag", "f", bool),
)
def main(data_size='all', force=False):
    download('de', force)


if __name__ == '__main__':
    plac.call(main)



from __future__ import unicode_literals
import pytest


@pytest.fixture
def close_puncts():
    return [')', ']', '}', '*']


def test_close(close_puncts, en_tokenizer):
    word_str = 'Hello'
    for p in close_puncts:
        string = word_str + p
        tokens = en_tokenizer(string)
        assert len(tokens) == 2
        assert tokens[1].string == p
        assert tokens[0].string == word_str


def test_two_different_close(close_puncts, en_tokenizer):
    word_str = 'Hello'
    for p in close_puncts:
        string = word_str + p + "'"
        tokens = en_tokenizer(string)
        assert len(tokens) == 3
        assert tokens[0].string == word_str
        assert tokens[1].string == p
        assert tokens[2].string == "'"


def test_three_same_close(close_puncts, en_tokenizer):
    word_str = 'Hello'
    for p in close_puncts:
        string = word_str + p + p + p
        tokens = en_tokenizer(string)
        assert len(tokens) == 4
        assert tokens[0].string == word_str
        assert tokens[1].string == p


def test_double_end_quote(en_tokenizer):
    assert len(en_tokenizer("Hello''")) == 2
    assert len(en_tokenizer("''")) == 1



# coding: utf-8
from __future__ import unicode_literals

import pytest
import io
import pickle
import cloudpickle
import tempfile


# @pytest.mark.xfail
# def test_pickle(en_tokenizer):
#     file_ = io.BytesIO()
#     cloudpickle.dump(en_tokenizer, file_)
#     file_.seek(0)
#     loaded = pickle.load(file_)
#     assert loaded is not None


def test_no_word(en_tokenizer):
    tokens = en_tokenizer(u'')
    assert len(tokens) == 0


def test_single_word(en_tokenizer):
    tokens = en_tokenizer(u'hello')
    assert tokens[0].orth_ == 'hello'


def test_two_words(en_tokenizer):
    tokens = en_tokenizer('hello possums')
    assert len(tokens) == 2
    assert tokens[0].orth_ != tokens[1].orth_


def test_punct(en_tokenizer):
    tokens = en_tokenizer('hello, possums.')
    assert len(tokens) == 4
    assert tokens[0].orth_ == 'hello'
    assert tokens[1].orth_ == ','
    assert tokens[2].orth_ == 'possums'
    assert tokens[1].orth_ != 'hello'


def test_digits(en_tokenizer):
    tokens = en_tokenizer('The year: 1984.')
    assert len(tokens) == 5
    assert tokens[0].orth == en_tokenizer.vocab['The'].orth
    assert tokens[3].orth == en_tokenizer.vocab['1984'].orth


def test_contraction(en_tokenizer):
    tokens = en_tokenizer("don't giggle")
    assert len(tokens) == 3
    assert tokens[1].orth == en_tokenizer.vocab["n't"].orth
    tokens = en_tokenizer("i said don't!")
    assert len(tokens) == 5
    assert tokens[4].orth == en_tokenizer.vocab['!'].orth


def test_contraction_punct(en_tokenizer):
    tokens = en_tokenizer("(can't")
    assert len(tokens) == 3
    tokens = en_tokenizer("`ain't")
    assert len(tokens) == 3
    tokens = en_tokenizer('''"isn't''')
    assert len(tokens) == 3
    tokens = en_tokenizer("can't!")
    assert len(tokens) == 3


def test_sample(en_tokenizer):
    text = """Tributes pour in for late British Labour Party leader

Tributes poured in from around the world Thursday
to the late Labour Party leader John Smith, who died earlier from a massive
heart attack aged 55.

In Washington, the US State Department issued a statement regretting "the
untimely death" of the rapier-tongued Scottish barrister and parliamentarian.

"Mr. Smith, throughout his distinguished"""

    tokens = en_tokenizer(text)
    assert len(tokens) > 5


def test_cnts1(en_tokenizer):
    text = u"""The U.S. Army likes Shock and Awe."""
    tokens = en_tokenizer(text)
    assert len(tokens) == 8


def test_cnts2(en_tokenizer):
    text = u"""U.N. regulations are not a part of their concern."""
    tokens = en_tokenizer(text)
    assert len(tokens) == 10


def test_cnts3(en_tokenizer):
    text = u"“Isn't it?”"
    tokens = en_tokenizer(text)
    words = [t.orth_ for t in tokens]
    assert len(words) == 6


def test_cnts4(en_tokenizer):
    text = u"""Yes! "I'd rather have a walk", Ms. Comble sighed. """
    tokens = en_tokenizer(text)
    words = [t.orth_ for t in tokens]
    assert len(words) == 15


def test_cnts5(en_tokenizer):
    text = """'Me too!', Mr. P. Delaware cried. """
    tokens = en_tokenizer(text)
    assert len(tokens) == 11


@pytest.mark.xfail
def test_mr(en_tokenizer):
    text = """Today is Tuesday.Mr."""
    tokens = en_tokenizer(text)
    assert len(tokens) == 5
    assert [w.orth_ for w in tokens] == ['Today', 'is', 'Tuesday', '.', 'Mr.']


def test_cnts6(en_tokenizer):
    text = u'They ran about 10km.'
    tokens = en_tokenizer(text)
    words = [t.orth_ for t in tokens]
    assert len(words) == 6

def test_bracket_period(en_tokenizer):
    text = u'(And a 6a.m. run through Washington Park).'
    tokens = en_tokenizer(text)
    assert tokens[len(tokens) - 1].orth_ == u'.'


def test_ie(en_tokenizer):
    text = u"It's mediocre i.e. bad."
    tokens = en_tokenizer(text)
    assert len(tokens) == 6
    assert tokens[3].orth_ == "i.e."


def test_two_whitespace(en_tokenizer):
    orig_str = u'there are 2 spaces after this  '
    tokens = en_tokenizer(orig_str)
    assert repr(tokens.text_with_ws) == repr(orig_str)


@pytest.mark.xfail
def test_em_dash_infix(en_tokenizer):
    # Re Issue #225
    tokens = en_tokenizer('''Will this road take me to Puddleton?\u2014No, '''
                          '''you'll have to walk there.\u2014Ariel.''')
    assert tokens[6].text == 'Puddleton'
    assert tokens[7].text == '?'
    assert tokens[8].text == '\u2014'

#def test_cnts7():
#    text = 'But then the 6,000-year ice age came...'
#    tokens = EN.tokenize(text)
#    assert len(tokens) == 10



class RegexMerger(object):
    def __init__(self, regexes):
        self.regexes = regexes

    def __call__(self, tokens):
        for tag, entity_type, regex in self.regexes:
            for m in regex.finditer(tokens.string):
                tokens.merge(m.start(), m.end(), tag, m.group(), entity_type)



from __future__ import unicode_literals, print_function

from os import path

from ..language import Language
from ..attrs import LANG
from . import language_data


class Italian(Language):
    lang = 'it'
    
    class Defaults(Language.Defaults):
        tokenizer_exceptions = dict(language_data.TOKENIZER_EXCEPTIONS)
        lex_attr_getters = dict(Language.Defaults.lex_attr_getters)
        lex_attr_getters[LANG] = lambda text: 'it'
        
        prefixes = tuple(language_data.TOKENIZER_PREFIXES)
        
        suffixes = tuple(language_data.TOKENIZER_SUFFIXES)
        
        infixes = tuple(language_data.TOKENIZER_INFIXES)

        tag_map = dict(language_data.TAG_MAP)

        stop_words = set(language_data.STOP_WORDS)




from __future__ import unicode_literals, print_function

from os import path

from ..language import Language
from . import language_data
from .. import util
from ..lemmatizer import Lemmatizer
from ..vocab import Vocab
from ..tokenizer import Tokenizer
from ..attrs import LANG


class English(Language):
    lang = 'en'

    class Defaults(Language.Defaults):
        lex_attr_getters = dict(Language.Defaults.lex_attr_getters)
        lex_attr_getters[LANG] = lambda text: 'en'

        tokenizer_exceptions = dict(language_data.TOKENIZER_EXCEPTIONS)
 
        prefixes = tuple(language_data.TOKENIZER_PREFIXES)

        suffixes = tuple(language_data.TOKENIZER_SUFFIXES)

        infixes = tuple(language_data.TOKENIZER_INFIXES)

        tag_map = dict(language_data.TAG_MAP)

        stop_words = set(language_data.STOP_WORDS)



from __future__ import unicode_literals
from spacy.en import English
from spacy.attrs import IS_ALPHA, IS_ASCII, IS_DIGIT, IS_LOWER, IS_PUNCT
from spacy.attrs import IS_SPACE, IS_TITLE, IS_UPPER, LIKE_URL, LIKE_NUM
from spacy.attrs import IS_STOP

import pytest
import numpy


@pytest.mark.models
def test_strings(EN):
    tokens = EN(u'Give it back! He pleaded.')
    token = tokens[0]
    assert token.orth_ == 'Give'
    assert token.text == 'Give'
    assert token.text_with_ws == 'Give '
    assert token.lower_ == 'give'
    assert token.shape_ == 'Xxxx'
    assert token.prefix_ == 'G'
    assert token.suffix_ == 'ive'
    assert token.lemma_ == 'give'
    assert token.pos_ == 'VERB'
    assert token.tag_ == 'VB'
    assert token.dep_ == 'ROOT'


def test_flags(EN):
    tokens = EN(u'Give it back! He pleaded.')
    token = tokens[0]
 
    assert token.check_flag(IS_ALPHA)
    assert not token.check_flag(IS_DIGIT)
    # TODO: Test more of these, esp. if a bug is found


def test_single_token_string(EN):
    tokens = EN(u'foobar')
    assert tokens[0].text == 'foobar'


def test_str_builtin(EN):
    tokens = EN('one two')
    assert str(tokens[0]) == u'one'
    assert str(tokens[1]) == u'two'


@pytest.mark.models
def test_is_properties(EN):
    Hi, comma, my, email, is_, addr = EN(u'Hi, my email is test@me.com')
    assert Hi.is_title
    assert Hi.is_alpha
    assert not Hi.is_digit
    assert comma.is_punct
    assert email.is_ascii
    assert not email.like_url
    assert is_.is_lower
    assert addr.like_email
    assert addr.is_oov
    assert not Hi.is_oov

@pytest.mark.models
def test_vectors(EN):
    apples, oranges, oov = EN(u'apples oranges ldskbjlsdkbflzdfbl')
    assert apples.has_vector
    assert oranges.has_vector
    assert not oov.has_vector
    assert apples.similarity(oranges) > apples.similarity(oov)
    assert apples.similarity(oranges) == oranges.similarity(apples)
    assert sum(apples.vector) != sum(oranges.vector)
    assert numpy.isclose(
                apples.vector_norm,
                numpy.sqrt(numpy.dot(apples.vector, apples.vector)))
    
@pytest.mark.models
def test_ancestors(EN):
    # the structure of this sentence depends on the English annotation scheme
    tokens = EN(u'Yesterday I saw a dog that barked loudly.')
    ancestors = [ t.orth_ for t in tokens[6].ancestors ]
    assert ancestors == ['dog','saw']
    ancestors = [ t.orth_ for t in tokens[1].ancestors ]
    assert ancestors == ['saw']
    ancestors = [ t.orth_ for t in tokens[2].ancestors ]
    assert ancestors == []

    assert tokens[2].is_ancestor_of(tokens[7])
    assert not tokens[6].is_ancestor_of(tokens[2])


@pytest.mark.models
def test_head_setter(EN):
    # the structure of this sentence depends on the English annotation scheme
    yesterday, i, saw, a, dog, that, barked, loudly, dot = EN(u'Yesterday I saw a dog that barked loudly.')
    assert barked.n_lefts == 1
    assert barked.n_rights == 1
    assert barked.left_edge == that
    assert barked.right_edge == loudly

    assert dog.n_lefts == 1
    assert dog.n_rights == 1
    assert dog.left_edge == a
    assert dog.right_edge == loudly
    
    assert a.n_lefts == 0
    assert a.n_rights == 0
    assert a.left_edge == a
    assert a.right_edge == a

    assert saw.left_edge == yesterday
    assert saw.right_edge == dot

    barked.head = a

    assert barked.n_lefts == 1
    assert barked.n_rights == 1
    assert barked.left_edge == that
    assert barked.right_edge == loudly

    assert a.n_lefts == 0
    assert a.n_rights == 1
    assert a.left_edge == a
    assert a.right_edge == loudly

    assert dog.n_lefts == 1
    assert dog.n_rights == 0
    assert dog.left_edge == a
    assert dog.right_edge == loudly

    assert saw.left_edge == yesterday
    assert saw.right_edge == dot

    yesterday.head = that

    assert that.left_edge == yesterday
    assert barked.left_edge == yesterday
    assert a.left_edge == yesterday
    assert dog.left_edge == yesterday
    assert saw.left_edge == yesterday



from __future__ import unicode_literals
import pytest


@pytest.mark.models
def test_subtrees(EN):
    sent = EN('The four wheels on the bus turned quickly')
    wheels = sent[2]
    bus = sent[5]
    assert len(list(wheels.lefts)) == 2
    assert len(list(wheels.rights)) == 1
    assert len(list(wheels.children)) == 3
    assert len(list(bus.lefts)) == 1
    assert len(list(bus.rights)) == 0
    assert len(list(bus.children)) == 1

    assert len(list(wheels.subtree)) == 6



# -*- coding: utf-8 -*-
import pytest


def test_print_doc(EN):
    try:
        doc = EN(u'I sat down for coffee at the coffee store')
        print(doc)
    except Exception:
        pytest.fail("Printing failed")


def test_repr_doc(EN):
    try:
        doc = EN(u'I sat down for coffee at the coffee store')
        print(repr(doc))
    except Exception:
        pytest.fail("Printing failed")


def test_print_doc_unicode(EN):
    try:
        doc = EN(u'I sat down for coffee at the café')
        print(doc)
    except Exception:
        pytest.fail("Printing failed")


def test_repr_doc_unicode(EN):
    try:
        doc = EN(u'I sat down for coffee at the café')
        print(repr(doc))
    except Exception:
        pytest.fail("Printing failed")


def test_print_span(EN):
    try:
        span = EN(u'I sat down for coffee at the coffee store')[-3:]
        print(span)
    except Exception:
        pytest.fail("Printing failed")


def test_repr_span(EN):
    try:
        span = EN(u'I sat down for coffee at the coffee store')[-3:]
        print(repr(span))
    except Exception:
        pytest.fail("Printing failed")


def test_print_span_unicode(EN):
    try:
        span = EN(u'I sat down for coffee at the café')[-3:]
        print(span)
    except Exception:
        pytest.fail("Printing failed")


def test_repr_span_unicode(EN):
    try:
        span = EN(u'I sat down for coffee at the café')[-3:]
        print(repr(span))
    except Exception:
        pytest.fail("Printing failed")


def test_print_token(EN):
    try:
        token = EN(u'I sat down for coffee at the coffee store')[-1]
        print(token)
    except Exception:
        pytest.fail("Printing failed")


def test_repr_token(EN):
    try:
        token = EN(u'I sat down for coffee at the coffee store')[-1]
        print(repr(token))
    except Exception:
        pytest.fail("Printing failed")


def test_print_token_unicode(EN):
    try:
        token = EN(u'I sat down for coffee at the café')[-1]
        print(token)
    except Exception:
        pytest.fail("Printing failed")


def test_repr_token_unicode(EN):
    try:
        token = EN(u'I sat down for coffee at the café')[-1]
        print(repr(token))
    except Exception:
        pytest.fail("Printing failed")



from spacy.deprecated import detokenize

def test_punct():
    tokens = 'Pierre Vinken , 61 years old .'.split()
    detoks = [(0,), (1, 2), (3,), (4,), (5, 6)]
    token_rules = ('<SEP>,', '<SEP>.')
    assert detokenize(token_rules, tokens) == detoks


def test_contractions():
    tokens = "I ca n't even".split()
    detoks = [(0,), (1, 2), (3,)]
    token_rules = ("ca<SEP>n't",)
    assert detokenize(token_rules, tokens) == detoks


def test_contractions_punct():
    tokens = "I ca n't !".split()
    detoks = [(0,), (1, 2, 3)]
    token_rules = ("ca<SEP>n't", '<SEP>!')
    assert detokenize(token_rules, tokens) == detoks



from __future__ import unicode_literals

import pytest
from spacy.tokens import Doc
from spacy.syntax.nonproj import PseudoProjectivity


@pytest.mark.models
def test_single_period(EN):
    string = 'A test sentence.'
    words = EN(string)
    assert len(words) == 4
    assert len(list(words.sents)) == 1
    assert sum(len(sent) for sent in words.sents) == len(words)


@pytest.mark.models
def test_single_no_period(EN):
    string = 'A test sentence'
    words = EN(string)
    assert len(words) == 3
    assert len(list(words.sents)) == 1
    assert sum(len(sent) for sent in words.sents) == len(words)


@pytest.mark.models
def test_single_exclamation(EN):
    string = 'A test sentence!'
    words = EN(string)
    assert len(words) == 4
    assert len(list(words.sents)) == 1
    assert sum(len(sent) for sent in words.sents) == len(words)


@pytest.mark.models
def test_single_question(EN):
    string = 'A test sentence?'
    words = EN(string, tag=False, parse=True)
    assert len(words) == 4
    assert len(list(words.sents)) == 1
    assert sum(len(sent) for sent in words.sents) == len(words)


@pytest.mark.models
def test_sentence_breaks(EN):
    doc = EN.tokenizer.tokens_from_list(u'This is a sentence . This is another one .'.split(' '))
    EN.tagger(doc)
    with EN.parser.step_through(doc) as stepwise:
        assert EN.parser.moves.is_valid(stepwise.stcls,'B-ROOT')
        stepwise.transition('L-nsubj')
        assert EN.parser.moves.is_valid(stepwise.stcls,'B-ROOT')
        stepwise.transition('S')
        assert EN.parser.moves.is_valid(stepwise.stcls,'B-ROOT')
        stepwise.transition('L-det')
        assert EN.parser.moves.is_valid(stepwise.stcls,'B-ROOT')
        stepwise.transition('R-attr')
        assert EN.parser.moves.is_valid(stepwise.stcls,'B-ROOT')
        stepwise.transition('D')
        assert EN.parser.moves.is_valid(stepwise.stcls,'B-ROOT')
        stepwise.transition('R-punct')
        assert EN.parser.moves.is_valid(stepwise.stcls,'B-ROOT')
        stepwise.transition('B-ROOT')
        assert EN.parser.moves.is_valid(stepwise.stcls,'B-ROOT')
        stepwise.transition('L-nsubj')
        assert EN.parser.moves.is_valid(stepwise.stcls,'B-ROOT')
        stepwise.transition('S')
        assert EN.parser.moves.is_valid(stepwise.stcls,'B-ROOT')
        stepwise.transition('L-attr')
        assert EN.parser.moves.is_valid(stepwise.stcls,'B-ROOT')
        stepwise.transition('R-attr')
        assert EN.parser.moves.is_valid(stepwise.stcls,'B-ROOT')
        stepwise.transition('D')
        assert EN.parser.moves.is_valid(stepwise.stcls,'B-ROOT')
        stepwise.transition('R-punct')
    assert len(list(doc.sents)) == 2
    for tok in doc:
        assert tok.dep != 0 or tok.is_space
    assert [ tok.head.i for tok in doc ] == [1,1,3,1,1,6,6,8,6,6]


def apply_transition_sequence(model, doc, sequence):
    with model.parser.step_through(doc) as stepwise:
        for transition in sequence:
            stepwise.transition(transition)


@pytest.mark.models
def test_sbd_serialization_projective(EN):
    """
    test that before and after serialization, the sentence boundaries are the same.
    """

    example = EN.tokenizer.tokens_from_list(u"I bought a couch from IKEA. It was n't very comfortable .".split(' '))
    EN.tagger(example)
    apply_transition_sequence(EN, example, ['L-nsubj','S','L-det','R-dobj','D','R-prep','R-pobj','B-ROOT','L-nsubj','R-neg','D','S','L-advmod','R-acomp','D','R-punct'])

    example_serialized = Doc(EN.vocab).from_bytes(example.to_bytes())

    assert example.to_bytes() == example_serialized.to_bytes()
    assert [s.text for s in example.sents] == [s.text for s in example_serialized.sents]


def test_sbd_empty_string(EN):
    '''Test Issue #309: SBD fails on empty string
    '''
    doc = EN(u' ')
    doc.is_parsed = True
    assert len(doc) == 1
    sents = list(doc.sents)
    assert len(sents) == 1


# TODO:
# @pytest.mark.models
# def test_sbd_serialization_nonprojective(DE):
#     """
#     test that before and after serialization, the sentence boundaries are the same in a non-projective sentence.
#     """
#     example = EN.tokenizer.tokens_from_list(u"Den Mann hat Peter nicht gesehen . Er war zu langsam .".split(' '))
#     EN.tagger(example)
#     apply_transition_sequence(EN, example, ['L-nk','L-oa||oc','R-sb','D','S','L-ng','B-ROOT','L-nsubj','R-neg','D','S','L-advmod','R-acomp','D','R-punct'])
#     print [(t.dep_,t.head.i) for t in example]

#     example_serialized = Doc(EN.vocab).from_bytes(example.to_bytes())

#     assert example.to_bytes() == example_serialized.to_bytes()
#     assert [s.text for s in example.sents] == [s.text for s in example_serialized.sents]
    















"""Test the Token.conjuncts property"""
from __future__ import unicode_literals

import pytest


def orths(tokens):
    return [t.orth_ for t in tokens]


#def test_simple_two(EN):
#    tokens = EN('I lost money and pride.', tag=True, parse=True)
#    pride = tokens[4]
#    assert orths(pride.conjuncts) == ['money', 'pride']
#    money = tokens[2]
#    assert orths(money.conjuncts) == ['money', 'pride']


#def test_comma_three(EN):
#    tokens = EN('I found my wallet, phone and keys.')
#    keys = tokens[-2]
#    assert orths(keys.conjuncts) == ['wallet', 'phone', 'keys']
#    wallet = tokens[3]
#    assert orths(wallet.conjuncts) == ['wallet', 'phone', 'keys']


# This is failing due to parse errors
#def test_and_three():
#    tokens = NLU('I found my wallet and phone and keys.')
#    keys = tokens[-2]
#    assert orths(keys.conjuncts) == ['wallet', 'phone', 'keys']
#    wallet = tokens[3]
#    assert orths(wallet.conjuncts) == ['wallet', 'phone', 'keys']



from __future__ import unicode_literals
from spacy.en import English

import pytest

from spacy.en import English

@pytest.fixture
def morph_exc():
    return {
            'PRP$': {'his': {'L': '-PRP-', 'person': 3, 'case': 2}},
           }


@pytest.mark.models
def test_load_exc(morph_exc):
    # Do this local as we want to modify it
    nlp =  English()
    nlp.vocab.morphology.load_morph_exceptions(morph_exc)
    tokens = nlp('I like his style.', tag=True, parse=False)
    his = tokens[2]
    assert his.tag_ == 'PRP$'
    assert his.lemma_ == '-PRP-'



# coding: utf-8
from __future__ import unicode_literals

import pytest

from spacy import attrs


def test_attr_of_token(EN):
    text = u'An example sentence.'
    tokens = EN(text, tag=True, parse=False)
    example = EN.vocab[u'example']
    assert example.orth != example.shape
    feats_array = tokens.to_array((attrs.ORTH, attrs.SHAPE))
    assert feats_array[0][0] != feats_array[0][1]


@pytest.mark.models
def test_tag(EN):
    text = u'A nice sentence.'
    tokens = EN(text)
    assert tokens[0].tag != tokens[1].tag != tokens[2].tag != tokens[3].tag
    feats_array = tokens.to_array((attrs.ORTH, attrs.TAG))
    assert feats_array[0][1] == tokens[0].tag
    assert feats_array[1][1] == tokens[1].tag
    assert feats_array[2][1] == tokens[2].tag
    assert feats_array[3][1] == tokens[3].tag


@pytest.mark.models
def test_dep(EN):
    text = u'A nice sentence.'
    tokens = EN(text)
    feats_array = tokens.to_array((attrs.ORTH, attrs.DEP))
    assert feats_array[0][1] == tokens[0].dep
    assert feats_array[1][1] == tokens[1].dep
    assert feats_array[2][1] == tokens[2].dep
    assert feats_array[3][1] == tokens[3].dep



IDS = {
    "": NULL_ATTR,
    "IS_ALPHA": IS_ALPHA,
    "IS_ASCII": IS_ASCII,
    "IS_DIGIT": IS_DIGIT,
    "IS_LOWER": IS_LOWER,
    "IS_PUNCT": IS_PUNCT,
    "IS_SPACE": IS_SPACE,
    "IS_TITLE": IS_TITLE,
    "IS_UPPER": IS_UPPER,
    "LIKE_URL": LIKE_URL,
    "LIKE_NUM": LIKE_NUM,
    "LIKE_EMAIL": LIKE_EMAIL,
    "IS_STOP": IS_STOP,
    "IS_OOV": IS_OOV,
    "IS_BRACKET": IS_BRACKET,
    "IS_QUOTE": IS_QUOTE,
    "IS_LEFT_PUNCT": IS_LEFT_PUNCT,
    "IS_RIGHT_PUNCT": IS_RIGHT_PUNCT,
    "FLAG18": FLAG18,
    "FLAG19": FLAG19,
    "FLAG20": FLAG20,
    "FLAG21": FLAG21,
    "FLAG22": FLAG22,
    "FLAG23": FLAG23,
    "FLAG24": FLAG24,
    "FLAG25": FLAG25,
    "FLAG26": FLAG26,
    "FLAG27": FLAG27,
    "FLAG28": FLAG28,
    "FLAG29": FLAG29,
    "FLAG30": FLAG30,
    "FLAG31": FLAG31,
    "FLAG32": FLAG32,
    "FLAG33": FLAG33,
    "FLAG34": FLAG34,
    "FLAG35": FLAG35,
    "FLAG36": FLAG36,
    "FLAG37": FLAG37,
    "FLAG38": FLAG38,
    "FLAG39": FLAG39,
    "FLAG40": FLAG40,
    "FLAG41": FLAG41,
    "FLAG42": FLAG42,
    "FLAG43": FLAG43,
    "FLAG44": FLAG44,
    "FLAG45": FLAG45,
    "FLAG46": FLAG46,
    "FLAG47": FLAG47,
    "FLAG48": FLAG48,
    "FLAG49": FLAG49,
    "FLAG50": FLAG50,
    "FLAG51": FLAG51,
    "FLAG52": FLAG52,
    "FLAG53": FLAG53,
    "FLAG54": FLAG54,
    "FLAG55": FLAG55,
    "FLAG56": FLAG56,
    "FLAG57": FLAG57,
    "FLAG58": FLAG58,
    "FLAG59": FLAG59,
    "FLAG60": FLAG60,
    "FLAG61": FLAG61,
    "FLAG62": FLAG62,
    "FLAG63": FLAG63,

    "ID": ID,
    "ORTH": ORTH,
    "LOWER": LOWER,
    "NORM": NORM,
    "SHAPE": SHAPE,
    "PREFIX": PREFIX,
    "SUFFIX": SUFFIX,

    "LENGTH": LENGTH,
    "CLUSTER": CLUSTER,
    "LEMMA": LEMMA,
    "POS": POS,
    "TAG": TAG,
    "DEP": DEP,
    "ENT_IOB": ENT_IOB,
    "ENT_TYPE": ENT_TYPE,
    "HEAD": HEAD,
    "SPACY": SPACY,
    "PROB": PROB,
    "LANG": LANG,
}

# ATTR IDs, in order of the symbol
NAMES = [key for key, value in sorted(IDS.items(), key=lambda item: item[1])]



from __future__ import unicode_literals

from spacy.orth import like_number


def test_digits():
    assert like_number('10')
    assert like_number('1')


def test_comma():
    assert like_number('10,000')
    assert like_number('10,00')
    assert like_number(',10')


def test_period():
    assert like_number('999.0')
    assert like_number('.99')


def test_fraction():
    assert like_number('1/2')
    assert not like_number('1/2/3')


def test_word():
    assert like_number('one')
    assert like_number('two')
    assert like_number('billion')


def test_not_number():
    assert not like_number('dog')
    assert not like_number(',')



# cython: infer_types=True
from __future__ import unicode_literals, absolute_import

cimport cython
from libc.string cimport memcpy
from libc.stdint cimport uint64_t

from murmurhash.mrmr cimport hash64, hash32

from preshed.maps cimport map_iter, key_t

from .typedefs cimport hash_t
from libc.stdint cimport uint32_t

try:
    import ujson as json
except ImportError:
    import json


cpdef hash_t hash_string(unicode string) except 0:
    chars = string.encode('utf8')
    return _hash_utf8(chars, len(chars))


cdef hash_t _hash_utf8(char* utf8_string, int length):
    return hash64(utf8_string, length, 1)


cdef uint32_t _hash32_utf8(char* utf8_string, int length):
    return hash32(utf8_string, length, 1)


cdef unicode _decode(const Utf8Str* string):
    cdef int i, length
    if string.s[0] < sizeof(string.s) and string.s[0] != 0:
        return string.s[1:string.s[0]+1].decode('utf8')
    elif string.p[0] < 255:
        return string.p[1:string.p[0]+1].decode('utf8')
    else:
        i = 0
        length = 0
        while string.p[i] == 255:
            i += 1
            length += 255
        length += string.p[i]
        i += 1
        return string.p[i:length + i].decode('utf8')


cdef Utf8Str _allocate(Pool mem, const unsigned char* chars, int length) except *:
    cdef int n_length_bytes
    cdef int i
    cdef Utf8Str string
    assert length != 0
    if length < sizeof(string.s):
        string.s[0] = <unsigned char>length
        memcpy(&string.s[1], chars, length)
        return string
    elif length < 255:
        string.p = <unsigned char*>mem.alloc(length + 1, sizeof(unsigned char))
        string.p[0] = length
        memcpy(&string.p[1], chars, length)
        assert string.s[0] >= sizeof(string.s) or string.s[0] == 0, string.s[0]
        return string
    else:
        i = 0
        n_length_bytes = (length // 255) + 1
        string.p = <unsigned char*>mem.alloc(length + n_length_bytes, sizeof(unsigned char))
        for i in range(n_length_bytes-1):
            string.p[i] = 255
        string.p[n_length_bytes-1] = length % 255
        memcpy(&string.p[n_length_bytes], chars, length)
        assert string.s[0] >= sizeof(string.s) or string.s[0] == 0, string.s[0]
        return string


cdef class StringStore:
    '''Map strings to and from integer IDs.'''
    def __init__(self, strings=None, freeze=False):
        '''Create the StringStore.

        Arguments:
            strings: A sequence of unicode strings to add to the store.
        '''
        self.mem = Pool()
        self._map = PreshMap()
        self._oov = PreshMap()
        self._resize_at = 10000
        self.c = <Utf8Str*>self.mem.alloc(self._resize_at, sizeof(Utf8Str))
        self.size = 1
        self.is_frozen = freeze
        if strings is not None:
            for string in strings:
                _ = self[string]

    property size:
        def __get__(self):
            return self.size -1

    def __len__(self):
        """The number of strings in the store.

        Returns:
            int The number of strings in the store.
        """
        return self.size-1

    def __getitem__(self, object string_or_id):
        """Retrieve a string from a given integer ID, or vice versa.
        
        Arguments:
            string_or_id (bytes or unicode or int):
                The value to encode.
        Returns:
            unicode or int: The value to retrieved.
        """
        if isinstance(string_or_id, basestring) and len(string_or_id) == 0:
            return 0
        elif string_or_id == 0:
            return u''

        cdef bytes byte_string
        cdef const Utf8Str* utf8str
        cdef uint64_t int_id
        cdef uint32_t oov_id
        if isinstance(string_or_id, (int, long)):
            int_id = string_or_id
            oov_id = string_or_id
            if int_id < <uint64_t>self.size:
                return _decode(&self.c[int_id])
            else:
                utf8str = <Utf8Str*>self._oov.get(oov_id)
                if utf8str is not NULL:
                    return _decode(utf8str)
                else:
                    raise IndexError(string_or_id)
        else:
            if isinstance(string_or_id, bytes):
                byte_string = <bytes>string_or_id
            elif isinstance(string_or_id, unicode):
                byte_string = (<unicode>string_or_id).encode('utf8')
            else:
                raise TypeError(type(string_or_id))
            utf8str = self._intern_utf8(byte_string, len(byte_string))
            if utf8str is NULL:
                # TODO: We need to use 32 bit here, for compatibility with the 
                # vocabulary values. This makes birthday paradox probabilities
                # pretty bad.
                # We could also get unlucky here, and hash into a value that
                # collides with the 'real' strings. 
                return _hash32_utf8(byte_string, len(byte_string))
            else:
                return utf8str - self.c

    def __contains__(self, unicode string not None):
        """Check whether a string is in the store.

        Arguments:
            string (unicode): The string to check.
        Returns bool:
            Whether the store contains the string.
        """
        if len(string) == 0:
            return True
        cdef hash_t key = hash_string(string)
        return self._map.get(key) is not NULL

    def __iter__(self):
        """Iterate over the strings in the store, in order.

        Yields: unicode A string in the store.
        """
        cdef int i
        for i in range(self.size):
            yield _decode(&self.c[i]) if i > 0 else u''
        # TODO: Iterate OOV here?

    def __reduce__(self):
        strings = [""]
        for i in range(1, self.size):
            string = &self.c[i]
            py_string = _decode(string)
            strings.append(py_string)
        return (StringStore, (strings,), None, None, None)

    def set_frozen(self, bint is_frozen):
        # TODO
        self.is_frozen = is_frozen

    def flush_oov(self):
        self._oov = PreshMap()

    cdef const Utf8Str* intern_unicode(self, unicode py_string):
        # 0 means missing, but we don't bother offsetting the index.
        cdef bytes byte_string = py_string.encode('utf8')
        return self._intern_utf8(byte_string, len(byte_string))

    @cython.final
    cdef const Utf8Str* _intern_utf8(self, char* utf8_string, int length):
        # TODO: This function's API/behaviour is an unholy mess...
        # 0 means missing, but we don't bother offsetting the index.
        cdef hash_t key = _hash_utf8(utf8_string, length)
        cdef Utf8Str* value = <Utf8Str*>self._map.get(key)
        if value is not NULL:
            return value
        value = <Utf8Str*>self._oov.get(key)
        if value is not NULL:
            return value
        if self.is_frozen:
            # OOV store uses 32 bit hashes. Pretty ugly :(
            key32 = _hash32_utf8(utf8_string, length)
            # Important: Make the OOV store own the memory. That way it's trivial
            # to flush them all.
            value = <Utf8Str*>self._oov.mem.alloc(1, sizeof(Utf8Str))
            value[0] = _allocate(self._oov.mem, <unsigned char*>utf8_string, length)
            self._oov.set(key32, value)
            return NULL

        if self.size == self._resize_at:
            self._realloc()
        self.c[self.size] = _allocate(self.mem, <unsigned char*>utf8_string, length)
        self._map.set(key, <void*>&self.c[self.size])
        self.size += 1
        return &self.c[self.size-1]

    def dump(self, file_):
        """Save the strings to a JSON file.

        Arguments:
            file_ (buffer): The file to save the strings.
        Returns:
            None
        """
        string_data = json.dumps(list(self))
        if not isinstance(string_data, unicode):
            string_data = string_data.decode('utf8')
        # TODO: OOV?
        file_.write(string_data)

    def load(self, file_):
        """Load the strings from a JSON file.

        Arguments:
            file_ (buffer): The file from which to load the strings.
        Returns:
            None
        """
        strings = json.load(file_)
        if strings == ['']:
            return None
        cdef unicode string
        for string in strings:
            # explicit None/len check instead of simple truth testing
            # (bug in Cython <= 0.23.4)
            if string is not None and len(string):
                self.intern_unicode(string)

    def _realloc(self):
        # We want to map straight to pointers, but they'll be invalidated if
        # we resize our array. So, first we remap to indices, then we resize,
        # then we can acquire the new pointers.
        cdef Pool tmp_mem = Pool()
        keys = <key_t*>tmp_mem.alloc(self.size, sizeof(key_t))
        cdef key_t key
        cdef void* value
        cdef const Utf8Str ptr
        cdef int i = 0
        cdef size_t offset
        while map_iter(self._map.c_map, &i, &key, &value):
            # Find array index with pointer arithmetic
            offset = ((<Utf8Str*>value) - self.c)
            keys[offset] = key
        
        self._resize_at *= 2
        cdef size_t new_size = self._resize_at * sizeof(Utf8Str)
        self.c = <Utf8Str*>self.mem.realloc(self.c, new_size)

        self._map = PreshMap(self.size)
        for i in range(self.size):
            if keys[i]:
                self._map.set(keys[i], &self.c[i])



import pytest
import os

from ..en import English
from ..de import German


@pytest.fixture(scope="session")
def EN():
    return English()

@pytest.fixture(scope="session")
def DE():
    return German()


def pytest_addoption(parser):
    parser.addoption("--models", action="store_true",
        help="include tests that require full models")
    parser.addoption("--vectors", action="store_true",
        help="include word vectors tests")
    parser.addoption("--slow", action="store_true",
        help="include slow tests")


def pytest_runtest_setup(item):
    for opt in ['models', 'vectors', 'slow']:
        if opt in item.keywords and not item.config.getoption("--%s" % opt):
            pytest.skip("need --%s option to run" % opt)



from libcpp.vector cimport vector
from libc.stdint cimport uint32_t
from libc.stdint cimport int64_t
from libc.stdint cimport int32_t
from libc.stdint cimport uint64_t

from .bits cimport BitArray, Code


cdef struct Node:
    int32_t left
    int32_t right


cdef class HuffmanCodec:
    cdef vector[Node] nodes
    cdef vector[Code] codes
    cdef Node root

    cdef readonly list leaves
    cdef readonly dict _map 
    
    cpdef int encode_int32(self, int32_t[:] msg, BitArray bits) except -1
    cpdef int decode_int32(self, BitArray bits, int32_t[:] msg) except -1



from __future__ import unicode_literals
import pytest

from spacy.attrs import DEP, HEAD
import numpy

from spacy.syntax.nonproj import ancestors, contains_cycle, is_nonproj_arc, is_nonproj_tree, PseudoProjectivity

def test_ancestors():
	tree = [1,2,2,4,5,2,2]
	cyclic_tree = [1,2,2,4,5,3,2]
	partial_tree = [1,2,2,4,5,None,2]
	multirooted_tree = [3,2,0,3,3,7,7,3,7,10,7,10,11,12,18,16,18,17,12,3]
	assert([ a for a in ancestors(3,tree) ] == [4,5,2])
	assert([ a for a in ancestors(3,cyclic_tree) ] == [4,5,3,4,5,3,4])
	assert([ a for a in ancestors(3,partial_tree) ] == [4,5,None])
	assert([ a for a in ancestors(17,multirooted_tree) ] == [])

def test_contains_cycle():
	tree = [1,2,2,4,5,2,2]
	cyclic_tree = [1,2,2,4,5,3,2]
	partial_tree = [1,2,2,4,5,None,2]
	multirooted_tree = [3,2,0,3,3,7,7,3,7,10,7,10,11,12,18,16,18,17,12,3]
	assert(contains_cycle(tree) == None)
	assert(contains_cycle(cyclic_tree) == set([3,4,5]))
	assert(contains_cycle(partial_tree) == None)
	assert(contains_cycle(multirooted_tree) == None)

def test_is_nonproj_arc():
	nonproj_tree = [1,2,2,4,5,2,7,4,2]
	partial_tree = [1,2,2,4,5,None,7,4,2]
	multirooted_tree = [3,2,0,3,3,7,7,3,7,10,7,10,11,12,18,16,18,17,12,3]
	assert(is_nonproj_arc(0,nonproj_tree) == False)
	assert(is_nonproj_arc(1,nonproj_tree) == False)
	assert(is_nonproj_arc(2,nonproj_tree) == False)
	assert(is_nonproj_arc(3,nonproj_tree) == False)
	assert(is_nonproj_arc(4,nonproj_tree) == False)
	assert(is_nonproj_arc(5,nonproj_tree) == False)
	assert(is_nonproj_arc(6,nonproj_tree) == False)
	assert(is_nonproj_arc(7,nonproj_tree) == True)
	assert(is_nonproj_arc(8,nonproj_tree) == False)
	assert(is_nonproj_arc(7,partial_tree) == False)
	assert(is_nonproj_arc(17,multirooted_tree) == False)
	assert(is_nonproj_arc(16,multirooted_tree) == True)

def test_is_nonproj_tree():
	proj_tree = [1,2,2,4,5,2,7,5,2]
	nonproj_tree = [1,2,2,4,5,2,7,4,2]
	partial_tree = [1,2,2,4,5,None,7,4,2]
	multirooted_tree = [3,2,0,3,3,7,7,3,7,10,7,10,11,12,18,16,18,17,12,3]
	assert(is_nonproj_tree(proj_tree) == False)
	assert(is_nonproj_tree(nonproj_tree) == True)
	assert(is_nonproj_tree(partial_tree) == False)
	assert(is_nonproj_tree(multirooted_tree) == True)


def deprojectivize(proj_heads, deco_labels, EN):
	slen = len(proj_heads)
	sent = EN.tokenizer.tokens_from_list(['whatever'] * slen)
	rel_proj_heads = [ head-i for i,head in enumerate(proj_heads) ]
	labelids = [ EN.vocab.strings[label] for label in deco_labels ]
	pairs = list(zip(rel_proj_heads,labelids))
	parse = numpy.asarray(pairs, dtype=numpy.int32)	
	sent.from_array([HEAD,DEP],parse)
	PseudoProjectivity.deprojectivize(sent)
	parse = sent.to_array([HEAD,DEP])
	deproj_heads = [ i+head for i,head in enumerate(parse[:,0]) ]
	undeco_labels = [ EN.vocab.strings[int(labelid)] for labelid in parse[:,1] ]
	return deproj_heads, undeco_labels


@pytest.mark.models
def test_pseudoprojectivity(EN):
	tree = [1,2,2]
	nonproj_tree = [1,2,2,4,5,2,7,4,2]
	labels = ['det','nsubj','root','det','dobj','aux','nsubj','acl','punct']
	nonproj_tree2 = [9,1,3,1,5,6,9,8,6,1,6,12,13,10,1]
	labels2 = ['advmod','root','det','nsubj','advmod','det','dobj','det','nmod','aux','nmod','advmod','det','amod','punct']

	assert(PseudoProjectivity.decompose('X||Y') == ('X','Y'))
	assert(PseudoProjectivity.decompose('X') == ('X',''))

	assert(PseudoProjectivity.is_decorated('X||Y') == True)
	assert(PseudoProjectivity.is_decorated('X') == False)

	PseudoProjectivity._lift(0,tree)
	assert(tree == [2,2,2])

	np_arc = PseudoProjectivity._get_smallest_nonproj_arc(nonproj_tree)
	assert(np_arc == 7)

	np_arc = PseudoProjectivity._get_smallest_nonproj_arc(nonproj_tree2)
	assert(np_arc == 10)

	proj_heads, deco_labels = PseudoProjectivity.projectivize(nonproj_tree,labels)
	assert(proj_heads == [1,2,2,4,5,2,7,5,2])
	assert(deco_labels == ['det','nsubj','root','det','dobj','aux','nsubj','acl||dobj','punct'])
	deproj_heads, undeco_labels = deprojectivize(proj_heads,deco_labels,EN)
	assert(deproj_heads == nonproj_tree)
	assert(undeco_labels == labels)

	proj_heads, deco_labels = PseudoProjectivity.projectivize(nonproj_tree2,labels2)
	assert(proj_heads == [1,1,3,1,5,6,9,8,6,1,9,12,13,10,1])
	assert(deco_labels == ['advmod||aux','root','det','nsubj','advmod','det','dobj','det','nmod','aux','nmod||dobj','advmod','det','amod','punct'])
	deproj_heads, undeco_labels = deprojectivize(proj_heads,deco_labels,EN)
	assert(deproj_heads == nonproj_tree2)
	assert(undeco_labels == labels2)

	# if decoration is wrong such that there is no head with the desired label
	# the structure is kept and the label is undecorated
	proj_heads = [1,2,2,4,5,2,7,5,2]
	deco_labels = ['det','nsubj','root','det','dobj','aux','nsubj','acl||iobj','punct']
	deproj_heads, undeco_labels = deprojectivize(proj_heads,deco_labels,EN)
	assert(deproj_heads == proj_heads)
	assert(undeco_labels == ['det','nsubj','root','det','dobj','aux','nsubj','acl','punct'])

	# if there are two potential new heads, the first one is chosen even if it's wrong
	proj_heads = [1,1,3,1,5,6,9,8,6,1,9,12,13,10,1]
	deco_labels = ['advmod||aux','root','det','aux','advmod','det','dobj','det','nmod','aux','nmod||dobj','advmod','det','amod','punct']
	deproj_heads, undeco_labels = deprojectivize(proj_heads,deco_labels,EN)
	assert(deproj_heads == [3,1,3,1,5,6,9,8,6,1,6,12,13,10,1])
	assert(undeco_labels == ['advmod','root','det','aux','advmod','det','dobj','det','nmod','aux','nmod','advmod','det','amod','punct'])


















import pytest

from spacy.serialize.packer import Packer
from spacy.attrs import ORTH, SPACY
from spacy.tokens import Doc
import math
import tempfile
import shutil
import os


@pytest.mark.models
def test_read_write(EN):
    doc1 = EN(u'This is a simple test. With a couple of sentences.')
    doc2 = EN(u'This is another test document.')

    try:
        tmp_dir = tempfile.mkdtemp()
        with open(os.path.join(tmp_dir, 'spacy_docs.bin'), 'wb') as file_:
            file_.write(doc1.to_bytes())
            file_.write(doc2.to_bytes())

        with open(os.path.join(tmp_dir, 'spacy_docs.bin'), 'rb') as file_:
            bytes1, bytes2 = Doc.read_bytes(file_)
            r1 = Doc(EN.vocab).from_bytes(bytes1)
            r2 = Doc(EN.vocab).from_bytes(bytes2)

        assert r1.string == doc1.string
        assert r2.string == doc2.string
    finally:
        shutil.rmtree(tmp_dir)


@pytest.mark.models
def test_left_right(EN):
    orig = EN(u'This is a simple test. With a couple of sentences.')
    result = Doc(orig.vocab).from_bytes(orig.to_bytes())

    for word in result:
        assert word.head.i == orig[word.i].head.i
        if word.head is not word:
            assert word.i in [w.i for w in word.head.children]
        for child in word.lefts:
            assert child.head.i == word.i
        for child in word.rights:
            assert child.head.i == word.i


@pytest.mark.models
def test_lemmas(EN):
    orig = EN(u'The geese are flying')
    result = Doc(orig.vocab).from_bytes(orig.to_bytes())
    the, geese, are, flying = result
    assert geese.lemma_ == 'goose'
    assert are.lemma_ == 'be'
    assert flying.lemma_ == 'fly'

 



from __future__ import unicode_literals
import spacy
from spacy.vocab import Vocab
from spacy.matcher import Matcher
from spacy.tokens.doc import Doc
from spacy.attrs import *
from spacy.pipeline import EntityRecognizer

import pytest


@pytest.fixture(scope="module")
def en_vocab():
    return spacy.get_lang_class('en').Defaults.create_vocab()


@pytest.fixture(scope="module")
def entity_recognizer(en_vocab):
    return EntityRecognizer(en_vocab, features=[(2,), (3,)])

@pytest.fixture
def animal(en_vocab):
    return nlp.vocab.strings[u"ANIMAL"]


@pytest.fixture
def doc(en_vocab, entity_recognizer):
    doc = Doc(en_vocab, words=[u"this",  u"is", u"a", u"lion"])
    entity_recognizer(doc)
    return doc


def test_set_ents_iob(doc):
    assert len(list(doc.ents)) == 0
    tags = [w.ent_iob_ for w in doc]
    assert tags == (['O'] * len(doc))
    doc.ents = [(doc.vocab.strings['ANIMAL'], 3, 4)]
    tags = [w.ent_iob_ for w in doc]
    assert tags == ['O', 'O', 'O', 'B']
    doc.ents = [(doc.vocab.strings['WORD'], 0, 2)]
    tags = [w.ent_iob_ for w in doc]
    assert tags == ['B', 'I', 'O', 'O']



"""Test that token.idx correctly computes index into the original string."""

from __future__ import unicode_literals

import pytest


def test_simple_punct(en_tokenizer):
    text = 'to walk, do foo'
    tokens = en_tokenizer(text)
    assert tokens[0].idx == 0
    assert tokens[1].idx == 3
    assert tokens[2].idx == 7
    assert tokens[3].idx == 9
    assert tokens[4].idx == 12


def test_complex_punct(en_tokenizer):
    text = 'Tom (D., Ill.)!'
    tokens = en_tokenizer(text)
    assert tokens[0].idx == 0
    assert len(tokens[0]) == 3
    assert tokens[1].idx == 4
    assert len(tokens[1]) == 1
    assert tokens[2].idx == 5
    assert len(tokens[2]) == 2
    assert tokens[3].idx == 7
    assert len(tokens[3]) == 1
    assert tokens[4].idx == 9
    assert len(tokens[4]) == 4
    assert tokens[5].idx == 13
    assert tokens[6].idx == 14



from __future__ import absolute_import
from __future__ import unicode_literals

import random
from .gold import GoldParse
from .scorer import Scorer
from .gold import merge_sents


class Trainer(object):
    '''Manage training of an NLP pipeline.'''
    def __init__(self, nlp, gold_tuples):
        self.nlp = nlp
        self.gold_tuples = gold_tuples

    def epochs(self, nr_epoch, augment_data=None, gold_preproc=False):
        def _epoch():
            for raw_text, paragraph_tuples in self.gold_tuples:
                if gold_preproc:
                    raw_text = None
                else:
                    paragraph_tuples = merge_sents(paragraph_tuples)
                if augment_data is not None:
                    raw_text, paragraph_tuples = augment_data(raw_text, paragraph_tuples)
                docs = self.make_docs(raw_text, paragraph_tuples)
                golds = self.make_golds(docs, paragraph_tuples)
                for doc, gold in zip(docs, golds):
                    yield doc, gold

        for itn in range(nr_epoch):
            random.shuffle(self.gold_tuples)
            yield _epoch()
 
    def update(self, doc, gold):
        for process in self.nlp.pipeline:
            if hasattr(process, 'update'):
                process.update(doc, gold)
            process(doc)
        return doc

    def evaluate(self, dev_sents, gold_preproc=False):
        scorer = Scorer()
        for raw_text, paragraph_tuples in dev_sents:
            if gold_preproc:
                raw_text = None
            else:
                paragraph_tuples = merge_sents(paragraph_tuples)
            docs = self.make_docs(raw_text, paragraph_tuples)
            golds = self.make_golds(docs, paragraph_tuples)
            for doc, gold in zip(docs, golds):
                for process in self.nlp.pipeline:
                    process(doc)
                scorer.score(doc, gold)
        return scorer

    def make_docs(self, raw_text, paragraph_tuples):
        if raw_text is not None:
            return [self.nlp.tokenizer(raw_text)]
        else:
            return [self.nlp.tokenizer.tokens_from_list(sent_tuples[0][1])
                    for sent_tuples in paragraph_tuples]

    def make_golds(self, docs, paragraph_tuples):
        if len(docs) == 1:
            return [GoldParse(docs[0], sent_tuples[0])
                    for sent_tuples in paragraph_tuples]
        else:
            return [GoldParse(doc, sent_tuples[0])
                    for doc, sent_tuples in zip(docs, paragraph_tuples)]



from libcpp.vector cimport vector

from preshed.maps cimport PreshMap
from cymem.cymem cimport Pool

from .typedefs cimport hash_t
from .structs cimport LexemeC, TokenC
from .strings cimport StringStore
from .tokens.doc cimport Doc
from .vocab cimport Vocab, LexemesOrTokens, _Cached


cdef class Tokenizer:
    cdef Pool mem
    cdef PreshMap _cache
    cdef PreshMap _specials
    cpdef readonly Vocab vocab

    cdef public object prefix_search
    cdef public object suffix_search
    cdef public object infix_finditer
    cdef object _rules

    cpdef Doc tokens_from_list(self, list strings)

    cdef int _try_cache(self, hash_t key, Doc tokens) except -1
    cdef int _tokenize(self, Doc tokens, unicode span, hash_t key) except -1
    cdef unicode _split_affixes(self, Pool mem, unicode string, vector[LexemeC*] *prefixes,
                             vector[LexemeC*] *suffixes)
    cdef int _attach_tokens(self, Doc tokens, unicode string,
                            vector[LexemeC*] *prefixes, vector[LexemeC*] *suffixes) except -1

    cdef int _save_cached(self, const TokenC* tokens, hash_t key, int n) except -1



from __future__ import unicode_literals
from spacy.attrs import HEAD
import pytest
import numpy


def test_merge_tokens(EN):
    tokens = EN(u'Los Angeles start.')
    tokens.from_array([HEAD], numpy.asarray([[1, 1, 0, -1]], dtype='int32').T)
    assert len(tokens) == 4
    assert tokens[0].head.orth_ == 'Angeles'
    assert tokens[1].head.orth_ == 'start'
    tokens.merge(0, len('Los Angeles'), 'NNP', 'Los Angeles', 'GPE')
    assert len(tokens) == 3
    assert tokens[0].orth_ == 'Los Angeles'
    assert tokens[0].head.orth_ == 'start'


@pytest.mark.models
def test_merge_heads(EN):
    tokens = EN(u'I found a pilates class near work.')
    assert len(tokens) == 8
    tokens.merge(tokens[3].idx, tokens[4].idx + len(tokens[4]), tokens[4].tag_,
                 'pilates class', 'O')
    assert len(tokens) == 7
    assert tokens[0].head.i == 1
    assert tokens[1].head.i == 1
    assert tokens[2].head.i == 3
    assert tokens[3].head.i == 1
    assert tokens[4].head.i in [1, 3]
    assert tokens[5].head.i == 4


@pytest.mark.models
def test_issue_54(EN):
    text = u'Talks given by women had a slightly higher number of questions asked (3.2$\pm$0.2) than talks given by men (2.6$\pm$0.1).'
    tokens = EN(text)


@pytest.mark.models
def test_np_merges(EN):
    text = u'displaCy is a parse tool built with Javascript'
    tokens = EN(text)
    assert tokens[4].head.i == 1
    tokens.merge(tokens[2].idx, tokens[4].idx + len(tokens[4]), u'NP', u'tool', u'O')
    assert tokens[2].head.i == 1
    tokens = EN('displaCy is a lightweight and modern dependency parse tree visualization tool built with CSS3 and JavaScript.')
    
    ents = [(e[0].idx, e[-1].idx + len(e[-1]), e.label_, e.lemma_)
            for e in tokens.ents]
    for start, end, label, lemma in ents:
        merged = tokens.merge(start, end, label, lemma, label)
        assert merged != None, (start, end, label, lemma) 

@pytest.mark.models
def test_entity_merge(EN):
    tokens = EN(u'Stewart Lee is a stand up comedian who lives in England and loves Joe Pasquale.\n')
    assert(len(tokens) == 17)
    for ent in tokens.ents:
        label, lemma, type_ = (ent.root.tag_, ent.root.lemma_, max(w.ent_type_ for w in ent))
        ent.merge(label, lemma, type_)
    # check looping is ok
    assert(len(tokens) == 15)


@pytest.mark.models
def test_sentence_update_after_merge(EN):
    tokens = EN(u'Stewart Lee is a stand up comedian. He lives in England and loves Joe Pasquale.')
    sent1, sent2 = list(tokens.sents)
    init_len = len(sent1)
    init_len2 = len(sent2)
    merge_me = tokens[0:2]
    merge_me.merge(u'none', u'none', u'none')
    merge_me2 = tokens[-2:]
    merge_me2.merge(u'none', u'none', u'none')
    assert(len(sent1) == init_len - 1)
    assert(len(sent2) == init_len2 - 1)


@pytest.mark.models
def test_subtree_size_check(EN):
    tokens = EN(u'Stewart Lee is a stand up comedian who lives in England and loves Joe Pasquale')
    sent1 = list(tokens.sents)[0]
    init_len = len(list(sent1.root.subtree))
    merge_me = tokens[0:2]
    merge_me.merge(u'none', u'none', u'none')
    assert(len(list(sent1.root.subtree)) == init_len - 1)



import re


_mw_prepositions = [
    'close to',
    'down by',
    'on the way to',
    'on my way to',
    'on my way',
    'on his way to',
    'on his way',
    'on her way to',
    'on her way',
    'on your way to',
    'on your way',
    'on our way to',
    'on our way',
    'on their way to',
    'on their way',
    'along the route from'
]


MW_PREPOSITIONS_RE = re.compile('|'.join(_mw_prepositions), flags=re.IGNORECASE)


TIME_RE = re.compile(
    '{colon_digits}|{colon_digits} ?{am_pm}?|{one_two_digits} ?({am_pm})'.format(
        colon_digits=r'[0-2]?[0-9]:[0-5][0-9](?::[0-5][0-9])?',
        one_two_digits=r'[0-2]?[0-9]',
        am_pm=r'[ap]\.?m\.?'))

DATE_RE = re.compile(
    '(?:this|last|next|the) (?:week|weekend|{days})'.format(
        days='Monday|Tuesday|Wednesday|Thursday|Friday|Saturday|Sunday'
    ))


MONEY_RE = re.compile('\$\d+(?:\.\d+)?|\d+ dollars(?: \d+ cents)?')


DAYS_RE = re.compile('Monday|Tuesday|Wednesday|Thursday|Friday|Saturday|Sunday')


REGEXES = [('IN', 'O', MW_PREPOSITIONS_RE), ('CD', 'TIME', TIME_RE),
           ('NNP', 'DATE', DATE_RE),
           ('NNP', 'DATE', DAYS_RE), ('CD', 'MONEY', MONEY_RE)]



cimport numpy as np

from .doc cimport Doc


cdef class Span:
    cdef readonly Doc doc
    cdef readonly int start
    cdef readonly int end
    cdef readonly int start_char
    cdef readonly int end_char
    cdef readonly int label

    cdef public _vector
    cdef public _vector_norm


    cpdef int _recalculate_indices(self) except -1



from numpy cimport ndarray
from ..vocab cimport Vocab
from ..structs cimport TokenC
from ..attrs cimport attr_id_t
from .doc cimport Doc


cdef class Token:
    cdef readonly Vocab vocab
    cdef TokenC* c
    cdef readonly int i
    cdef readonly Doc doc

    @staticmethod
    cdef inline Token cinit(Vocab vocab, const TokenC* token, int offset, Doc doc):
        if offset < 0 or offset >= doc.length:
            msg = "Attempt to access token at %d, max length %d"
            raise IndexError(msg % (offset, doc.length))
        if doc._py_tokens[offset] != None:
            return doc._py_tokens[offset]
        cdef Token self = Token.__new__(Token, vocab, doc, offset)
        doc._py_tokens[offset] = self
        return self

    cpdef bint check_flag(self, attr_id_t flag_id) except -1



from __future__ import unicode_literals
import pytest

from spacy.parts_of_speech import ADV


@pytest.mark.models
def test_prob(EN):
    tokens = EN(u'Give it back', parse=False)
    give = tokens[0]
    assert give.prob != 0



import pytest

import pickle
import cloudpickle

import io


#@pytest.mark.models
#def test_pickle(EN):
#    file_ = io.BytesIO()
#    cloudpickle.dump(EN.parser, file_)
#
#    file_.seek(0)
#
#    loaded = pickle.load(file_)
#



import pytest


@pytest.mark.models
def test_initial(EN):
    doc = EN.tokenizer(u'I ate the pizza with anchovies.')
    EN.tagger(doc)
    with EN.parser.step_through(doc) as stepwise:
        stepwise.transition('L-nsubj')
        stepwise.transition('S')
        stepwise.transition('L-det')
    assert doc[0].head.i == 1
    assert doc[1].head.i == 1
    assert doc[2].head.i == 3
    assert doc[3].head.i == 3
    assert doc



import cloudpickle
import io
import os
import pickle
import pytest
import tempfile

try:
    unicode
except NameError:
    unicode = str

# These cause the addition of temp files, that are then not deleted
#@pytest.mark.models
#def test_pickle_english(EN):
#    file_ = io.BytesIO()
#    cloudpickle.dump(EN, file_)
#
#    file_.seek(0)
#
#    loaded = pickle.load(file_)
#    assert loaded is not None
#
#@pytest.mark.models
#def test_cloudpickle_to_file(EN):
#    f = tempfile.NamedTemporaryFile(delete=False)
#    p = cloudpickle.CloudPickler(f)
#    p.dump(EN)
#    f.close()
#    loaded_en = cloudpickle.load(open(f.name, 'rb'))
#    os.unlink(f.name)
#    doc = loaded_en(unicode('test parse'))
#    assert len(doc) == 2



from os import path
from .lemmatizer import Lemmatizer

try:
    import ujson as json
except ImportError:
    import json

from .parts_of_speech import IDS as POS_IDS
from .parts_of_speech cimport ADJ, VERB, NOUN, PUNCT
from .attrs cimport IS_SPACE
from .lexeme cimport Lexeme


cdef class Morphology:
    def __init__(self, StringStore string_store, tag_map, lemmatizer):
        self.mem = Pool()
        self.strings = string_store
        self.tag_map = tag_map
        self.lemmatizer = lemmatizer
        self.n_tags = len(tag_map) + 1
        self.tag_names = tuple(sorted(tag_map.keys()))
        self.reverse_index = {}
        
        self.rich_tags = <RichTagC*>self.mem.alloc(self.n_tags, sizeof(RichTagC))
        for i, (tag_str, props) in enumerate(sorted(tag_map.items())):
            self.rich_tags[i].id = i
            self.rich_tags[i].name = self.strings[tag_str]
            self.rich_tags[i].morph = 0
            self.rich_tags[i].pos = POS_IDS[props['pos'].upper()]
            self.reverse_index[self.rich_tags[i].name] = i
        self._cache = PreshMapArray(self.n_tags)

    def __reduce__(self):
        return (Morphology, (self.strings, self.tag_map, self.lemmatizer), None, None)

    cdef int assign_tag(self, TokenC* token, tag) except -1:
        if isinstance(tag, basestring):
            tag_id = self.reverse_index[self.strings[tag]]
        else:
            tag_id = self.reverse_index[tag]
        self.assign_tag_id(token, tag_id)

    cdef int assign_tag_id(self, TokenC* token, int tag_id) except -1:
        if tag_id >= self.n_tags:
            raise ValueError("Unknown tag ID: %s" % tag_id)
        # TODO: It's pretty arbitrary to put this logic here. I guess the justification
        # is that this is where the specific word and the tag interact. Still,
        # we should have a better way to enforce this rule, or figure out why
        # the statistical model fails.
        # Related to Issue #220
        if Lexeme.c_check_flag(token.lex, IS_SPACE):
            tag_id = self.reverse_index[self.strings['SP']]
        analysis = <MorphAnalysisC*>self._cache.get(tag_id, token.lex.orth)
        if analysis is NULL:
            analysis = <MorphAnalysisC*>self.mem.alloc(1, sizeof(MorphAnalysisC))
            analysis.tag = self.rich_tags[tag_id]
            tag_str = self.strings[self.rich_tags[tag_id].name]
            analysis.lemma = self.lemmatize(analysis.tag.pos, token.lex.orth,
                                            **self.tag_map.get(tag_str, {}))
            self._cache.set(tag_id, token.lex.orth, analysis)
        token.lemma = analysis.lemma
        token.pos = analysis.tag.pos
        token.tag = analysis.tag.name
        token.morph = analysis.tag.morph

    cdef int assign_feature(self, uint64_t* morph, feature, value) except -1:
        pass

    def load_morph_exceptions(self, dict exc):
        # Map (form, pos) to (lemma, rich tag)
        cdef unicode pos_str
        cdef unicode form_str
        cdef unicode lemma_str
        cdef dict entries
        cdef dict props
        cdef int lemma
        cdef attr_t orth
        cdef attr_t tag_id
        cdef int pos
        cdef RichTagC rich_tag
        for tag_str, entries in exc.items():
            tag = self.strings[tag_str]
            tag_id = self.reverse_index[tag] 
            rich_tag = self.rich_tags[tag_id]
            for form_str, props in entries.items():
                cached = <MorphAnalysisC*>self.mem.alloc(1, sizeof(MorphAnalysisC))
                cached.tag = rich_tag
                orth = self.strings[form_str]
                for name_str, value_str in props.items():
                    if name_str == 'L':
                        cached.lemma = self.strings[value_str]
                    else:
                        self.assign_feature(&cached.tag.morph, name_str, value_str)
                if cached.lemma == 0:
                    cached.lemma = self.lemmatize(rich_tag.pos, orth,
                                                  self.tag_map.get(tag_str, {}))
                self._cache.set(tag_id, orth, <void*>cached)

    def lemmatize(self, const univ_pos_t univ_pos, attr_t orth, **morphology):
        cdef unicode py_string = self.strings[orth]
        if self.lemmatizer is None:
            return self.strings[py_string.lower()]
        if univ_pos not in (NOUN, VERB, ADJ, PUNCT):
            return self.strings[py_string.lower()]
        cdef set lemma_strings
        cdef unicode lemma_string
        lemma_strings = self.lemmatizer(py_string, univ_pos, **morphology)
        lemma_string = sorted(lemma_strings)[0]
        lemma = self.strings[lemma_string]
        return lemma

IDS = {
    "Animacy_anim": Animacy_anim,
    "Animacy_inam": Animacy_inam,
    "Aspect_freq": Aspect_freq,
    "Aspect_imp": Aspect_imp,
    "Aspect_mod": Aspect_mod,
    "Aspect_none": Aspect_none,
    "Aspect_perf": Aspect_perf,
    "Case_abe": Case_abe,
    "Case_abl": Case_abl,
    "Case_abs": Case_abs,
    "Case_acc": Case_acc,
    "Case_ade": Case_ade,
    "Case_all": Case_all,
    "Case_cau": Case_cau,
    "Case_com": Case_com,
    "Case_dat": Case_dat,
    "Case_del": Case_del,
    "Case_dis": Case_dis,
    "Case_ela": Case_ela,
    "Case_ess": Case_ess,
    "Case_gen": Case_gen,
    "Case_ill": Case_ill,
    "Case_ine": Case_ine,
    "Case_ins": Case_ins,
    "Case_loc": Case_loc,
    "Case_lat": Case_lat,
    "Case_nom": Case_nom,
    "Case_par": Case_par,
    "Case_sub": Case_sub,
    "Case_sup": Case_sup,
    "Case_tem": Case_tem,
    "Case_ter": Case_ter,
    "Case_tra": Case_tra,
    "Case_voc": Case_voc,
    "Definite_two": Definite_two,
    "Definite_def": Definite_def,
    "Definite_red": Definite_red,
    "Definite_ind": Definite_ind,
    "Degree_cmp": Degree_cmp,
    "Degree_comp": Degree_comp,
    "Degree_none": Degree_none,
    "Degree_pos": Degree_pos,
    "Degree_sup": Degree_sup,
    "Degree_abs": Degree_abs,
    "Degree_com": Degree_com,
    "Degree_dim ": Degree_dim, # du
    "Gender_com": Gender_com,
    "Gender_fem": Gender_fem,
    "Gender_masc": Gender_masc,
    "Gender_neut": Gender_neut,
    "Mood_cnd": Mood_cnd,
    "Mood_imp": Mood_imp,
    "Mood_ind": Mood_ind,
    "Mood_n": Mood_n,
    "Mood_pot": Mood_pot,
    "Mood_sub": Mood_sub,
    "Mood_opt": Mood_opt,
    "Negative_neg": Negative_neg,
    "Negative_pos": Negative_pos,
    "Negative_yes": Negative_yes,
    "Number_com": Number_com,
    "Number_dual": Number_dual,
    "Number_none": Number_none,
    "Number_plur": Number_plur,
    "Number_sing": Number_sing,
    "Number_ptan ": Number_ptan, # bg
    "Number_count ": Number_count, # bg
    "NumType_card": NumType_card,
    "NumType_dist": NumType_dist,
    "NumType_frac": NumType_frac,
    "NumType_gen": NumType_gen,
    "NumType_mult": NumType_mult,
    "NumType_none": NumType_none,
    "NumType_ord": NumType_ord,
    "NumType_sets": NumType_sets,
    "Person_one": Person_one,
    "Person_two": Person_two,
    "Person_three": Person_three,
    "Person_none": Person_none,
    "Poss_yes": Poss_yes,
    "PronType_advPart": PronType_advPart,
    "PronType_art": PronType_art,
    "PronType_default": PronType_default,
    "PronType_dem": PronType_dem,
    "PronType_ind": PronType_ind,
    "PronType_int": PronType_int,
    "PronType_neg": PronType_neg,
    "PronType_prs": PronType_prs,
    "PronType_rcp": PronType_rcp,
    "PronType_rel": PronType_rel,
    "PronType_tot": PronType_tot,
    "PronType_clit": PronType_clit,
    "PronType_exc ": PronType_exc, # es, ca, it, fa,
    "Reflex_yes": Reflex_yes,
    "Tense_fut": Tense_fut,
    "Tense_imp": Tense_imp,
    "Tense_past": Tense_past,
    "Tense_pres": Tense_pres,
    "VerbForm_fin": VerbForm_fin,
    "VerbForm_ger": VerbForm_ger,
    "VerbForm_inf": VerbForm_inf,
    "VerbForm_none": VerbForm_none,
    "VerbForm_part": VerbForm_part,
    "VerbForm_partFut": VerbForm_partFut,
    "VerbForm_partPast": VerbForm_partPast,
    "VerbForm_partPres": VerbForm_partPres,
    "VerbForm_sup": VerbForm_sup,
    "VerbForm_trans": VerbForm_trans,
    "VerbForm_gdv ": VerbForm_gdv, # la,
    "Voice_act": Voice_act,
    "Voice_cau": Voice_cau,
    "Voice_pass": Voice_pass,
    "Voice_mid ": Voice_mid, # gkc,
    "Voice_int ": Voice_int, # hb,
    "Abbr_yes ": Abbr_yes, # cz, fi, sl, U,
    "AdpType_prep ": AdpType_prep, # cz, U,
    "AdpType_post ": AdpType_post, # U,
    "AdpType_voc ": AdpType_voc, # cz,
    "AdpType_comprep ": AdpType_comprep, # cz,
    "AdpType_circ ": AdpType_circ, # U,
    "AdvType_man": AdvType_man,
    "AdvType_loc": AdvType_loc,
    "AdvType_tim": AdvType_tim,
    "AdvType_deg": AdvType_deg,
    "AdvType_cau": AdvType_cau,
    "AdvType_mod": AdvType_mod,
    "AdvType_sta": AdvType_sta,
    "AdvType_ex": AdvType_ex,
    "AdvType_adadj": AdvType_adadj,
    "ConjType_oper ": ConjType_oper, # cz, U,
    "ConjType_comp ": ConjType_comp, # cz, U,
    "Connegative_yes ": Connegative_yes, # fi,
    "Derivation_minen ": Derivation_minen, # fi,
    "Derivation_sti ": Derivation_sti, # fi,
    "Derivation_inen ": Derivation_inen, # fi,
    "Derivation_lainen ": Derivation_lainen, # fi,
    "Derivation_ja ": Derivation_ja, # fi,
    "Derivation_ton ": Derivation_ton, # fi,
    "Derivation_vs ": Derivation_vs, # fi,
    "Derivation_ttain ": Derivation_ttain, # fi,
    "Derivation_ttaa ": Derivation_ttaa, # fi,
    "Echo_rdp ": Echo_rdp, # U,
    "Echo_ech ": Echo_ech, # U,
    "Foreign_foreign ": Foreign_foreign, # cz, fi, U,
    "Foreign_fscript ": Foreign_fscript, # cz, fi, U,
    "Foreign_tscript ": Foreign_tscript, # cz, U,
    "Foreign_yes ": Foreign_yes, # sl,
    "Gender_dat_masc ": Gender_dat_masc, # bq, U,
    "Gender_dat_fem ": Gender_dat_fem, # bq, U,
    "Gender_erg_masc ": Gender_erg_masc, # bq,
    "Gender_erg_fem ": Gender_erg_fem, # bq,
    "Gender_psor_masc ": Gender_psor_masc, # cz, sl, U,
    "Gender_psor_fem ": Gender_psor_fem, # cz, sl, U,
    "Gender_psor_neut ": Gender_psor_neut, # sl,
    "Hyph_yes ": Hyph_yes, # cz, U,
    "InfForm_one ": InfForm_one, # fi,
    "InfForm_two ": InfForm_two, # fi,
    "InfForm_three ": InfForm_three, # fi,
    "NameType_geo ": NameType_geo, # U, cz,
    "NameType_prs ": NameType_prs, # U, cz,
    "NameType_giv ": NameType_giv, # U, cz,
    "NameType_sur ": NameType_sur, # U, cz,
    "NameType_nat ": NameType_nat, # U, cz,
    "NameType_com ": NameType_com, # U, cz,
    "NameType_pro ": NameType_pro, # U, cz,
    "NameType_oth ": NameType_oth, # U, cz,
    "NounType_com ": NounType_com, # U,
    "NounType_prop ": NounType_prop, # U,
    "NounType_class ": NounType_class, # U,
    "Number_abs_sing ": Number_abs_sing, # bq, U,
    "Number_abs_plur ": Number_abs_plur, # bq, U,
    "Number_dat_sing ": Number_dat_sing, # bq, U,
    "Number_dat_plur ": Number_dat_plur, # bq, U,
    "Number_erg_sing ": Number_erg_sing, # bq, U,
    "Number_erg_plur ": Number_erg_plur, # bq, U,
    "Number_psee_sing ": Number_psee_sing, # U,
    "Number_psee_plur ": Number_psee_plur, # U,
    "Number_psor_sing ": Number_psor_sing, # cz, fi, sl, U,
    "Number_psor_plur ": Number_psor_plur, # cz, fi, sl, U,
    "NumForm_digit ": NumForm_digit, # cz, sl, U,
    "NumForm_roman ": NumForm_roman, # cz, sl, U,
    "NumForm_word ": NumForm_word, # cz, sl, U,
    "NumValue_one ": NumValue_one, # cz, U,
    "NumValue_two ": NumValue_two, # cz, U,
    "NumValue_three ": NumValue_three, # cz, U,
    "PartForm_pres ": PartForm_pres, # fi,
    "PartForm_past ": PartForm_past, # fi,
    "PartForm_agt ": PartForm_agt, # fi,
    "PartForm_neg ": PartForm_neg, # fi,
    "PartType_mod ": PartType_mod, # U,
    "PartType_emp ": PartType_emp, # U,
    "PartType_res ": PartType_res, # U,
    "PartType_inf ": PartType_inf, # U,
    "PartType_vbp ": PartType_vbp, # U,
    "Person_abs_one ": Person_abs_one, # bq, U,
    "Person_abs_two ": Person_abs_two, # bq, U,
    "Person_abs_three ": Person_abs_three, # bq, U,
    "Person_dat_one ": Person_dat_one, # bq, U,
    "Person_dat_two ": Person_dat_two, # bq, U,
    "Person_dat_three ": Person_dat_three, # bq, U,
    "Person_erg_one ": Person_erg_one, # bq, U,
    "Person_erg_two ": Person_erg_two, # bq, U,
    "Person_erg_three ": Person_erg_three, # bq, U,
    "Person_psor_one ": Person_psor_one, # fi, U,
    "Person_psor_two ": Person_psor_two, # fi, U,
    "Person_psor_three ": Person_psor_three, # fi, U,
    "Polite_inf ": Polite_inf, # bq, U,
    "Polite_pol ": Polite_pol, # bq, U,
    "Polite_abs_inf ": Polite_abs_inf, # bq, U,
    "Polite_abs_pol ": Polite_abs_pol, # bq, U,
    "Polite_erg_inf ": Polite_erg_inf, # bq, U,
    "Polite_erg_pol ": Polite_erg_pol, # bq, U,
    "Polite_dat_inf ": Polite_dat_inf, # bq, U,
    "Polite_dat_pol ": Polite_dat_pol, # bq, U,
    "Prefix_yes ": Prefix_yes, # U,
    "PrepCase_npr ": PrepCase_npr, # cz,
    "PrepCase_pre ": PrepCase_pre, # U,
    "PunctSide_ini ": PunctSide_ini, # U,
    "PunctSide_fin ": PunctSide_fin, # U,
    "PunctType_peri ": PunctType_peri, # U,
    "PunctType_qest ": PunctType_qest, # U,
    "PunctType_excl ": PunctType_excl, # U,
    "PunctType_quot ": PunctType_quot, # U,
    "PunctType_brck ": PunctType_brck, # U,
    "PunctType_comm ": PunctType_comm, # U,
    "PunctType_colo ": PunctType_colo, # U,
    "PunctType_semi ": PunctType_semi, # U,
    "PunctType_dash ": PunctType_dash, # U,
    "Style_arch ": Style_arch, # cz, fi, U,
    "Style_rare ": Style_rare, # cz, fi, U,
    "Style_poet ": Style_poet, # cz, U,
    "Style_norm ": Style_norm, # cz, U,
    "Style_coll ": Style_coll, # cz, U,
    "Style_vrnc ": Style_vrnc, # cz, U,
    "Style_sing ": Style_sing, # cz, U,
    "Style_expr ": Style_expr, # cz, U,
    "Style_derg ": Style_derg, # cz, U,
    "Style_vulg ": Style_vulg, # cz, U,
    "Style_yes ": Style_yes, # fi, U,
    "StyleVariant_styleShort ": StyleVariant_styleShort, # cz,
    "StyleVariant_styleBound ": StyleVariant_styleBound, # cz, sl,
    "VerbType_aux ": VerbType_aux, # U,
    "VerbType_cop ": VerbType_cop, # U,
    "VerbType_mod ": VerbType_mod, # U,
    "VerbType_light ": VerbType_light, # U,
}


NAMES = [key for key, value in sorted(IDS.items(), key=lambda item: item[1])]



from __future__ import unicode_literals
import pytest


@pytest.fixture
def paired_puncts():
    return [('(', ')'),  ('[', ']'), ('{', '}'), ('*', '*')]


def test_token(paired_puncts, en_tokenizer):
    word_str = 'Hello'
    for open_, close_ in paired_puncts:
        string = open_ + word_str + close_
        tokens = en_tokenizer(string)
        assert len(tokens) == 3
        assert tokens[0].orth_ == open_
        assert tokens[1].orth_ == word_str
        assert tokens[2].orth_ == close_


def test_two_different(paired_puncts, en_tokenizer):
    word_str = 'Hello'
    for open_, close_ in paired_puncts:
        string = "`" + open_ + word_str + close_ + "'"
        tokens = en_tokenizer(string)
        assert len(tokens) == 5
        assert tokens[0].orth_ == "`"
        assert tokens[1].orth_ == open_
        assert tokens[2].orth_ == word_str
        assert tokens[2].orth_ == word_str
        assert tokens[3].orth_ == close_
        assert tokens[4].orth_ == "'"



from __future__ import unicode_literals
import pytest

from spacy.orth import is_alpha
from spacy.orth import is_digit
from spacy.orth import is_punct
from spacy.orth import is_space
from spacy.orth import is_ascii
from spacy.orth import is_upper
from spacy.orth import is_lower
from spacy.orth import is_title


@pytest.fixture
def words():
    return ["1997", "19.97", "hello9", "Hello", "HELLO", "Hello9", "\n", "!",
            "!d", "\nd"]


def test_is_alpha(words):
    assert not is_alpha(words[0])
    assert not is_alpha(words[1])
    assert not is_alpha(words[2])
    assert is_alpha(words[3])
    assert is_alpha(words[4])
    assert not is_alpha(words[5])
    assert not is_alpha(words[6])
    assert not is_alpha(words[7])
    assert not is_alpha(words[8])
    assert not is_alpha(words[9])


def test_is_digit(words):
    assert is_digit(words[0])
    assert not is_digit(words[1])
    assert not is_digit(words[2])
    assert not is_digit(words[3])
    assert not is_digit(words[4])
    assert not is_digit(words[5])
    assert not is_digit(words[6])
    assert not is_digit(words[7])
    assert not is_digit(words[8])
    assert not is_digit(words[9])


def test_is_quote(words):
    pass


def test_is_bracket(words):
    pass


def test_is_left_bracket(words):
    pass

def test_is_right_bracket(words):
    pass



from __future__ import unicode_literals
import pytest

from spacy.strings import StringStore
from spacy.matcher import *
from spacy.attrs import LOWER
from spacy.tokens.doc import Doc
from spacy.vocab import Vocab
from spacy.en import English


@pytest.fixture
def matcher():
    patterns = {
        'JS': ['PRODUCT', {}, [[{'ORTH': 'JavaScript'}]]],
        'GoogleNow':  ['PRODUCT', {}, [[{'ORTH': 'Google'}, {'ORTH': 'Now'}]]],
        'Java':       ['PRODUCT', {}, [[{'LOWER': 'java'}]]],
    }
    return Matcher(Vocab(lex_attr_getters=English.Defaults.lex_attr_getters), patterns)


def test_compile(matcher):
    assert matcher.n_patterns == 3


def test_no_match(matcher):
    doc = Doc(matcher.vocab, words=['I', 'like', 'cheese', '.'])
    assert matcher(doc) == []


def test_match_start(matcher):
    doc = Doc(matcher.vocab, words=['JavaScript', 'is', 'good'])
    assert matcher(doc) == [(matcher.vocab.strings['JS'],
                             matcher.vocab.strings['PRODUCT'], 0, 1)]


def test_match_end(matcher):
    doc = Doc(matcher.vocab, words=['I', 'like', 'java'])
    assert matcher(doc) == [(doc.vocab.strings['Java'],
                             doc.vocab.strings['PRODUCT'], 2, 3)]


def test_match_middle(matcher):
    doc = Doc(matcher.vocab, words=['I', 'like', 'Google', 'Now', 'best'])
    assert matcher(doc) == [(doc.vocab.strings['GoogleNow'],
                             doc.vocab.strings['PRODUCT'], 2, 4)]


def test_match_multi(matcher):
    doc = Doc(matcher.vocab, words='I like Google Now and java best'.split())
    assert matcher(doc) == [(doc.vocab.strings['GoogleNow'],
                             doc.vocab.strings['PRODUCT'], 2, 4),
                            (doc.vocab.strings['Java'],
                             doc.vocab.strings['PRODUCT'], 5, 6)]

def test_match_zero(matcher):
    matcher.add('Quote', '', {}, [
        [
            {'ORTH': '"'},
            {'OP': '!', 'IS_PUNCT': True},
            {'OP': '!', 'IS_PUNCT': True},
            {'ORTH': '"'}
        ]])
    doc = Doc(matcher.vocab, words='He said , " some words " ...'.split())
    assert len(matcher(doc)) == 1
    doc = Doc(matcher.vocab, words='He said , " some three words " ...'.split())
    assert len(matcher(doc)) == 0
    matcher.add('Quote', '', {}, [
        [
            {'ORTH': '"'},
            {'IS_PUNCT': True},
            {'IS_PUNCT': True},
            {'IS_PUNCT': True},
            {'ORTH': '"'}
        ]])
    assert len(matcher(doc)) == 0


def test_match_zero_plus(matcher):
    matcher.add('Quote', '', {}, [
        [
            {'ORTH': '"'},
            {'OP': '*', 'IS_PUNCT': False},
            {'ORTH': '"'}
        ]])
    doc = Doc(matcher.vocab, words='He said , " some words " ...'.split())
    assert len(matcher(doc)) == 1


#@pytest.mark.models
#def test_match_preserved(EN):
#    patterns = {
#        'JS': ['PRODUCT', {}, [[{'ORTH': 'JavaScript'}]]],
#        'GoogleNow':  ['PRODUCT', {}, [[{'ORTH': 'Google'}, {'ORTH': 'Now'}]]],
#        'Java':       ['PRODUCT', {}, [[{'LOWER': 'java'}]]],
#    }
#    matcher = Matcher(EN.vocab, patterns)
#    doc = EN.tokenizer('I like java.')
#    EN.tagger(doc)
#    assert len(doc.ents) == 0
#    doc = EN.tokenizer('I like java.')
#    doc.ents += tuple(matcher(doc))
#    assert len(doc.ents) == 1
#    EN.tagger(doc)
#    EN.entity(doc)
#    assert len(doc.ents) == 1



import pathlib

from .util import set_lang_class, get_lang_class
from .about import __version__

from . import en
from . import de
from . import zh
from . import es
from . import it
from . import fr
from . import pt


try:
    basestring
except NameError:
    basestring = str


set_lang_class(en.English.lang, en.English)
set_lang_class(de.German.lang, de.German)
set_lang_class(es.Spanish.lang, es.Spanish)
set_lang_class(pt.Portuguese.lang, pt.Portuguese)
set_lang_class(fr.French.lang, fr.French)
set_lang_class(it.Italian.lang, it.Italian)
set_lang_class(zh.Chinese.lang, zh.Chinese)


def load(name, **overrides):
    target_name, target_version = util.split_data_name(name)
    data_path = overrides.get('path', util.get_data_path())
    if target_name == 'en' and 'add_vectors' not in overrides:
        if 'vectors' in overrides:
            vec_path = util.match_best_version(overrides['vectors'], None, data_path)
            if vec_path is None: 
                raise IOError(
                    'Could not load data pack %s from %s' % (overrides['vectors'], data_path))

        else:
            vec_path = util.match_best_version('en_glove_cc_300_1m_vectors', None, data_path)
        if vec_path is not None:
            vec_path = vec_path / 'vocab' / 'vec.bin'
            overrides['add_vectors'] = lambda vocab: vocab.load_vectors_from_bin_loc(vec_path)
    path = util.match_best_version(target_name, target_version, data_path)
    cls = get_lang_class(target_name)
    return cls(path=path, **overrides)



'''Test the possibly temporary workaround of flushing the stringstore of OOV words.'''
import pytest

from ...strings import StringStore


def test_oov():
    strings = StringStore()
    a = strings[u'a']
    b = strings[u'b']

    assert a == 1
    assert b == 2
    strings.set_frozen(True)
    c = strings[u'c']
    assert c >= 4
    c_ = strings[c]
    assert c_ == u'c'
    strings.flush_oov()
    with pytest.raises(IndexError):
        c_ = strings[c]



"""Test entries in the tokenization special-case interacting with prefix
and suffix punctuation."""
from __future__ import unicode_literals
import pytest


def test_no_special(en_tokenizer):
    assert len(en_tokenizer("(can)")) == 3


def test_no_punct(en_tokenizer):
    assert len(en_tokenizer("can't")) == 2


def test_prefix(en_tokenizer):
    assert len(en_tokenizer("(can't")) == 3


def test_suffix(en_tokenizer):
    assert len(en_tokenizer("can't)")) == 3


def test_wrap(en_tokenizer):
    assert len(en_tokenizer("(can't)")) == 4


def test_uneven_wrap(en_tokenizer):
    assert len(en_tokenizer("(can't?)")) == 5


def test_prefix_interact(en_tokenizer):
    assert len(en_tokenizer("U.S.")) == 1
    assert len(en_tokenizer("us.")) == 2
    assert len(en_tokenizer("(U.S.")) == 2


def test_suffix_interact(en_tokenizer):
    assert len(en_tokenizer("U.S.)")) == 2


def test_even_wrap_interact(en_tokenizer):
    assert len(en_tokenizer("(U.S.)")) == 3


def test_uneven_wrap_interact(en_tokenizer):
    assert len(en_tokenizer("(U.S.?)")) == 4



from __future__ import unicode_literals
import pytest
from spacy.attrs import HEAD
import numpy


@pytest.mark.xfail
def test_example_war_and_peace(nlp):
    # from spacy.en import English
    from spacy._doc_examples import download_war_and_peace

    unprocessed_unicode = download_war_and_peace()

    # nlp = English()
    # TODO: ImportError: No module named _doc_examples
    doc = nlp(unprocessed_unicode)


def test_main_entry_point(nlp):
    # from spacy.en import English
    # nlp = English()
    doc = nlp('Some text.') # Applies tagger, parser, entity
    doc = nlp('Some text.', parse=False) # Applies tagger and entity, not parser
    doc = nlp('Some text.', entity=False) # Applies tagger and parser, not entity
    doc = nlp('Some text.', tag=False) # Does not apply tagger, entity or parser
    doc = nlp('') # Zero-length tokens, not an error
    # doc = nlp(b'Some text') <-- Error: need unicode
    doc = nlp(b'Some text'.decode('utf8')) # Encode to unicode first.


@pytest.mark.models
def test_sentence_spans(nlp):
    # from spacy.en import English
    # nlp = English()
    doc = nlp("This is a sentence. Here's another...")
    assert [s.root.orth_ for s in doc.sents] == ["is", "'s"]


@pytest.mark.models
def test_entity_spans(nlp):
    # from spacy.en import English
    # nlp = English()
    tokens = nlp('Mr. Best flew to New York on Saturday morning.')
    ents = list(tokens.ents)
    assert ents[0].label == 346
    assert ents[0].label_ == 'PERSON'
    assert ents[0].orth_ == 'Best'
    assert ents[0].string == ents[0].string


@pytest.mark.models
def test_noun_chunk_spans(nlp):
    # from spacy.en import English
    # nlp = English()
    doc = nlp('The sentence in this example has three noun chunks.')
    for chunk in doc.noun_chunks:
        print(chunk.label, chunk.orth_, '<--', chunk.root.head.orth_)

    # NP The sentence <-- has
    # NP this example <-- in
    # NP three noun chunks <-- has


@pytest.mark.models
def test_count_by(nlp):
    # from spacy.en import English, attrs
    # nlp = English()
    import numpy
    from spacy import attrs
    tokens = nlp('apple apple orange banana')
    assert tokens.count_by(attrs.ORTH) == {3699: 2, 3750: 1, 5965: 1}
    assert repr(tokens.to_array([attrs.ORTH])) == repr(numpy.array([[3699],
                                                        [3699],
                                                        [3750],
                                                        [5965]], dtype=numpy.int32))

@pytest.mark.models
def test_read_bytes(nlp):
    from spacy.tokens.doc import Doc
    loc = 'test_serialize.bin'
    with open(loc, 'wb') as file_:
        file_.write(nlp(u'This is a document.').to_bytes())
        file_.write(nlp(u'This is another.').to_bytes())
    docs = []
    with open(loc, 'rb') as file_:
        for byte_string in Doc.read_bytes(file_):
            docs.append(Doc(nlp.vocab).from_bytes(byte_string))
    assert len(docs) == 2


def test_token_span(doc):
    span = doc[4:6]
    token = span[0]
    assert token.i == 4


@pytest.mark.models
def test_example_i_like_new_york1(nlp):
    toks = nlp('I like New York in Autumn.')


@pytest.fixture
def toks(nlp):
    doc = nlp('I like New York in Autumn.')
    doc.from_array([HEAD], numpy.asarray([[1, 0, 1, -2, -3, -1, -5]], dtype='int32').T)
    return doc


def test_example_i_like_new_york2(toks):
    i, like, new, york, in_, autumn, dot = range(len(toks))


@pytest.fixture
def tok(toks, tok):
    i, like, new, york, in_, autumn, dot = range(len(toks))
    return locals()[tok]


@pytest.fixture
def new(toks):
    return tok(toks, "new")


@pytest.fixture
def york(toks):
    return tok(toks, "york")


@pytest.fixture
def autumn(toks):
    return tok(toks, "autumn")


@pytest.fixture
def dot(toks):
    return tok(toks, "dot")


def test_example_i_like_new_york3(toks, new, york):
    assert toks[new].head.orth_ == 'York'
    assert toks[york].head.orth_ == 'like'


def test_example_i_like_new_york4(toks, new, york):
    new_york = toks[new:york+1]
    assert new_york.root.orth_ == 'York'


def test_example_i_like_new_york5(toks, autumn, dot):
    assert toks[autumn].head.orth_ == 'in'
    assert toks[dot].head.orth_ == 'like'
    autumn_dot = toks[autumn:]
    assert autumn_dot.root.orth_ == 'Autumn'


def test_navigating_the_parse_tree_lefts(doc):
    # TODO: where does the span object come from?
    span = doc[:2]
    lefts = [span.doc[i] for i in range(0, span.start)
             if span.doc[i].head in span]


def test_navigating_the_parse_tree_rights(doc):
    span = doc[:2]
    rights = [span.doc[i] for i in range(span.end, len(span.doc))
              if span.doc[i].head in span]


def test_string_store(doc):
    string_store = doc.vocab.strings
    for i, string in enumerate(string_store):
        assert i == string_store[string]



import pytest

@pytest.mark.models
def test_token_vector(EN):
    token = EN(u'Apples and oranges')[0]
    token.vector
    token.vector_norm

@pytest.mark.models
def test_lexeme_vector(EN):
    lexeme = EN.vocab[u'apples']
    lexeme.vector
    lexeme.vector_norm


@pytest.mark.models
def test_doc_vector(EN):
    doc = EN(u'Apples and oranges')
    doc.vector
    doc.vector_norm

@pytest.mark.models
def test_span_vector(EN):
    span = EN(u'Apples and oranges')[0:2]
    span.vector
    span.vector_norm

@pytest.mark.models
def test_token_token_similarity(EN):
    apples, oranges = EN(u'apples oranges')
    assert apples.similarity(oranges) == oranges.similarity(apples)
    assert 0.0 < apples.similarity(oranges) < 1.0
    

@pytest.mark.models
def test_token_lexeme_similarity(EN):
    apples = EN(u'apples')
    oranges = EN.vocab[u'oranges']
    assert apples.similarity(oranges) == oranges.similarity(apples)
    assert 0.0 < apples.similarity(oranges) < 1.0
 

@pytest.mark.models
def test_token_span_similarity(EN):
    doc = EN(u'apples orange juice')
    apples = doc[0]
    oranges = doc[1:3]
    assert apples.similarity(oranges) == oranges.similarity(apples)
    assert 0.0 < apples.similarity(oranges) < 1.0
 

@pytest.mark.models
def test_token_doc_similarity(EN):
    doc = EN(u'apples orange juice')
    apples = doc[0]
    assert apples.similarity(doc) == doc.similarity(apples)
    assert 0.0 < apples.similarity(doc) < 1.0
 

@pytest.mark.models
def test_lexeme_span_similarity(EN):
    doc = EN(u'apples orange juice')
    apples = EN.vocab[u'apples']
    span = doc[1:3]
    assert apples.similarity(span) == span.similarity(apples)
    assert 0.0 < apples.similarity(span) < 1.0


@pytest.mark.models
def test_lexeme_lexeme_similarity(EN):
    apples = EN.vocab[u'apples']
    oranges = EN.vocab[u'oranges']
    assert apples.similarity(oranges) == oranges.similarity(apples)
    assert 0.0 < apples.similarity(oranges) < 1.0
 

@pytest.mark.models
def test_lexeme_doc_similarity(EN):
    doc = EN(u'apples orange juice')
    apples = EN.vocab[u'apples']
    assert apples.similarity(doc) == doc.similarity(apples)
    assert 0.0 < apples.similarity(doc) < 1.0
 

@pytest.mark.models
def test_span_span_similarity(EN):
    doc = EN(u'apples orange juice')
    apples = doc[0:2]
    oj = doc[1:3]
    assert apples.similarity(oj) == oj.similarity(apples)
    assert 0.0 < apples.similarity(oj) < 1.0
 

@pytest.mark.models
def test_span_doc_similarity(EN):
    doc = EN(u'apples orange juice')
    apples = doc[0:2]
    oj = doc[1:3]
    assert apples.similarity(doc) == doc.similarity(apples)
    assert 0.0 < apples.similarity(doc) < 1.0
 

@pytest.mark.models
def test_doc_doc_similarity(EN):
    apples = EN(u'apples and apple pie')
    oranges = EN(u'orange juice')
    assert apples.similarity(oranges) == apples.similarity(oranges)
    assert 0.0 < apples.similarity(oranges) < 1.0
 



from __future__ import unicode_literals
import pytest
import os


@pytest.fixture(scope='session')
def nlp():
    from spacy.en import English
    if os.environ.get('SPACY_DATA'):
        data_dir = os.environ.get('SPACY_DATA')
    else:
        data_dir = True
    return English(path=data_dir)


@pytest.fixture()
def doc(nlp):
    for word in ['Hello', ',', 'world', '.', 'Here', 'are', 'two', 'sentences', '.']:
        _ = nlp.vocab[word]
    return nlp('Hello, world. Here are two sentences.')



from __future__ import unicode_literals
from os import path
import io

import pytest


@pytest.fixture
def sun_text():
    with io.open(path.join(path.dirname(__file__), '..', 'sun.txt'), 'r',
                 encoding='utf8') as file_:
        text = file_.read()
    return text


@pytest.mark.models
def test_consistency(EN, sun_text):
    tokens = EN(sun_text)
    for head in tokens:
        for child in head.lefts:
            assert child.head is head
        for child in head.rights:
            assert child.head is head


@pytest.mark.models
def test_child_consistency(EN, sun_text):
    tokens = EN(sun_text)

    lefts = {}
    rights = {}
    for head in tokens:
        assert head.i not in lefts
        lefts[head.i] = set()
        for left in head.lefts:
            lefts[head.i].add(left.i)
        assert head.i not in rights
        rights[head.i] = set()
        for right in head.rights:
            rights[head.i].add(right.i)
    for head in tokens:
        assert head.n_rights == len(rights[head.i])
        assert head.n_lefts == len(lefts[head.i])
    for child in tokens:
        if child.i < child.head.i:
            assert child.i in lefts[child.head.i]
            assert child.i not in rights[child.head.i]
            lefts[child.head.i].remove(child.i)
        elif child.i > child.head.i:
            assert child.i in rights[child.head.i]
            assert child.i not in lefts[child.head.i]
            rights[child.head.i].remove(child.i)
    for head_index, children in lefts.items():
        assert not children
    for head_index, children in rights.items():
        assert not children


@pytest.mark.models
def test_edges(EN, sun_text):
    tokens = EN(sun_text)
    for token in tokens:
        subtree = list(token.subtree)
        debug = '\t'.join((token.orth_, token.left_edge.orth_, subtree[0].orth_))
        assert token.left_edge == subtree[0], debug
        debug = '\t'.join((token.orth_, token.right_edge.orth_, subtree[-1].orth_, token.right_edge.head.orth_))
        assert token.right_edge == subtree[-1], debug



# -*- coding: utf-8 -*-
"""Sphinx doctest is just too hard. Manually paste doctest examples here"""
import pytest

#@pytest.mark.models
#def test_1():
#    import spacy.en
#    from spacy.parts_of_speech import ADV
#    # Load the pipeline, and call it with some text.
#    nlp = spacy.en.English()
#    tokens = nlp(u"‘Give it back,’ he pleaded abjectly, ‘it’s mine.’",
#                tag=True, parse=False)
#    o = u''.join(tok.string.upper() if tok.pos == ADV else tok.string for tok in tokens)
#    assert u"‘Give it BACK,’ he pleaded ABJECTLY, ‘it’s mine.’"
#
#    o = nlp.vocab[u'back'].prob
#    assert o == -7.033305644989014
#    o = nlp.vocab[u'not'].prob
#    assert o == -5.332601070404053
#    o = nlp.vocab[u'quietly'].prob
#    assert o == -11.994928359985352
#
#
#@pytest.mark.m
#def test2():
#    import spacy.en
#    from spacy.parts_of_speech import ADV
#    nlp = spacy.en.English()
#    # Find log probability of Nth most frequent word
#    probs = [lex.prob for lex in nlp.vocab]
#    probs.sort()
#    is_adverb = lambda tok: tok.pos == ADV and tok.prob < probs[-1000]
#    tokens = nlp(u"‘Give it back,’ he pleaded abjectly, ‘it’s mine.’")
#    o = u''.join(tok.string.upper() if is_adverb(tok) else tok.string for tok in tokens)
#    o == u'‘Give it back,’ he pleaded ABJECTLY, ‘it’s mine.’'
#
#@pytest.mark.models
#def test3():
#    import spacy.en
#    from spacy.parts_of_speech import ADV
#    nlp = spacy.en.English()
#    # Find log probability of Nth most frequent word
#    probs = [lex.prob for lex in nlp.vocab]
#    probs.sort()
#    is_adverb = lambda tok: tok.pos == ADV and tok.prob < probs[-1000]
#    tokens = nlp(u"‘Give it back,’ he pleaded abjectly, ‘it’s mine.’")
#    o = u''.join(tok.string.upper() if is_adverb(tok) else tok.string for tok in tokens)
#    assert o == u'‘Give it back,’ he pleaded ABJECTLY, ‘it’s mine.’'
#
#    pleaded = tokens[7]
#    assert pleaded.repvec.shape == (300,)
#    o = pleaded.repvec[:5]
#    assert sum(o) != 0
#    from numpy import dot
#    from numpy.linalg import norm
#
#    cosine = lambda v1, v2: dot(v1, v2) / (norm(v1) * norm(v2))
#    words = [w for w in nlp.vocab if w.is_lower and w.has_repvec]
#    words.sort(key=lambda w: cosine(w.repvec, pleaded.repvec))
#    words.reverse()
#    o = [w.orth_ for w in words[0:20]]
#    assert o == [u'pleaded', u'pled', u'plead', u'confessed', u'interceded',
#                 u'pleads', u'testified', u'conspired', u'motioned', u'demurred',
#                 u'countersued', u'remonstrated', u'begged', u'apologised',
#                 u'consented', u'acquiesced', u'petitioned', u'quarreled',
#                 u'appealed', u'pleading']
#    o = [w.orth_ for w in words[50:60]]
#    assert o == [u'martialed', u'counselled', u'bragged',
#                 u'backtracked', u'caucused', u'refiled', u'dueled', u'mused',
#                 u'dissented', u'yearned']
#    o = [w.orth_ for w in words[100:110]]
#    assert o == [u'acquits', u'cabled', u'ducked', u'sentenced',
#                 u'gaoled', u'perjured', u'absconded', u'bargained', u'overstayed',
#                 u'clerked']
#    
#    #o = [w.orth_ for w in words[1000:1010]]
#    #assert o == [u'scorned', u'baled', u'righted', u'requested', u'swindled',
#    #             u'posited', u'firebombed', u'slimed', u'deferred', u'sagged']
#    #o = [w.orth_ for w in words[50000:50010]]
#    #assert o == [u'fb', u'ford', u'systems', u'puck', u'anglers', u'ik', u'tabloid',
#    #             u'dirty', u'rims', u'artists']



from .syntax.parser cimport Parser
from .syntax.ner cimport BiluoPushDown
from .syntax.arc_eager cimport ArcEager
from .tagger cimport Tagger


cdef class EntityRecognizer(Parser):
    pass


cdef class DependencyParser(Parser):
    pass
